classdef AnalysisBrightness < matlab.apps.AppBase

    % Properties that correspond to app components
    properties (Access = public)
        UIFigure                     matlab.ui.Figure
        StevensconstantLabel         matlab.ui.control.Label
        EstimationscanbeeditedLabel  matlab.ui.control.Label
        LinearregressionLabel        matlab.ui.control.Label
        StevensexponentLabel         matlab.ui.control.Label
        bEditField                   matlab.ui.control.NumericEditField
        bEditFieldLabel              matlab.ui.control.Label
        aEditField                   matlab.ui.control.NumericEditField
        aEditFieldLabel              matlab.ui.control.Label
        StevensparametersLabel       matlab.ui.control.Label
        BEditField                   matlab.ui.control.NumericEditField
        BEditFieldLabel              matlab.ui.control.Label
        AEditField                   matlab.ui.control.NumericEditField
        AEditFieldLabel              matlab.ui.control.Label
        SumlogLlogBrEditField        matlab.ui.control.NumericEditField
        SumlogLlogBrEditFieldLabel   matlab.ui.control.Label
        SumlogL2EditField            matlab.ui.control.NumericEditField
        SumlogL2EditFieldLabel       matlab.ui.control.Label
        SumlogBrEditField            matlab.ui.control.NumericEditField
        SumlogBrEditFieldLabel       matlab.ui.control.Label
        SumlogLEditField             matlab.ui.control.NumericEditField
        SumlogLEditFieldLabel        matlab.ui.control.Label
        nEditField                   matlab.ui.control.NumericEditField
        nEditFieldLabel              matlab.ui.control.Label
        SaveanalysisButton           matlab.ui.control.Button
        SerranoPedrazaI2023Label     matlab.ui.control.Label
        LoaddatafileButton           matlab.ui.control.Button
        UITable                      matlab.ui.control.Table
        AnalysisofbrightnessestimationLabel  matlab.ui.control.Label
        UIAxes2                      matlab.ui.control.UIAxes
        UIAxes                       matlab.ui.control.UIAxes
    end

    
    properties (Access = public)
        filename; % Description
    end
    
    properties (Access = private)
        CallingApp; % Main app 
        Indices;
    end
    
    
    
    methods (Access = private)
        
        function updateplot(app)
            
            %%Dr. Ignacio Serrano-Pedraza 2023
            %%%With this program you can analyze the data from the
            %%%experiment Experiment_Stevens_Brightness.m

            % Get Table UI component data
            t = app.UITable.DisplayData;            
            Trials=t.Trials;
            Estimations=t.Estimations;
            Luminance_test=t.Luminance_test;
            logBr=log10(Estimations);
            logL=log10(Luminance_test);
            
            logL2=logL.^2;
            logLlogBr=logL.*logBr;

            T=table(Trials,Luminance_test,Estimations,logBr,logL,logL2,logLlogBr);

            app.UITable.Data = T;

             x2 = log10(t.Luminance_test);
             y2 = log10(t.Estimations);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
              app.nEditField.Value= length(x2);
              app.SumlogLEditField.Value=sum(x2);
              app.SumlogBrEditField.Value=sum(y2);
              app.SumlogL2EditField.Value=sum(x2.^2);
              app.SumlogLlogBrEditField.Value=sum(x2.*y2);

              %%%Linear regression
              app.AEditField.Value=(app.nEditField.Value*app.SumlogLlogBrEditField.Value-app.SumlogLEditField.Value*app.SumlogBrEditField.Value)...
                  /(app.nEditField.Value*app.SumlogL2EditField.Value-app.SumlogLEditField.Value^2);
              app.BEditField.Value=mean(y2)-app.AEditField.Value*mean(x2);


              app.aEditField.Value=app.AEditField.Value;
              app.bEditField.Value=10^app.BEditField.Value;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % % Plot data 
             
              loglog(app.UIAxes,Luminance_test,Estimations,'ko')
              hold(app.UIAxes,'on');
              x4=[1:300];
              y4=app.bEditField.Value*x4.^(app.aEditField.Value);
              loglog(app.UIAxes,x4,y4,'k-','Linewidth',2)
              axis(app.UIAxes,[1 300 1 300])
              hold(app.UIAxes,'off');

              plot(app.UIAxes2,Luminance_test,Estimations,'ko')
              hold(app.UIAxes2,'on');
              x4=[1:300];
              y4=app.bEditField.Value*x4.^(app.aEditField.Value);
              plot(app.UIAxes2,x4,y4,'k-','Linewidth',2)
              axis(app.UIAxes2,[1 300 1 300])
              hold(app.UIAxes2,'off');

                if isempty(app.Indices)==0
              x3 = Luminance_test(unique(app.Indices(:,1)));
              y3 = Estimations(unique(app.Indices(:,1)));
              loglog(app.UIAxes,Luminance_test,Estimations,'ko','MarkerSize',8)
              hold(app.UIAxes,'on');
              loglog(app.UIAxes,x3,y3,'ro','Linewidth',2,'MarkerSize',8)
              hold(app.UIAxes,'on');
              x4=[1:300];
              y4=app.bEditField.Value*x4.^(app.aEditField.Value);
              loglog(app.UIAxes,x4,y4,'k-','Linewidth',2)
              axis(app.UIAxes2,[1 300 1 300])
              hold(app.UIAxes,'off');

              x3 = Luminance_test(unique(app.Indices(:,1)));
              y3 = Estimations(unique(app.Indices(:,1)));
              plot(app.UIAxes2,Luminance_test,Estimations,'ko','MarkerSize',8)
              hold(app.UIAxes2,'on');
              plot(app.UIAxes2,x3,y3,'ro','Linewidth',2,'MarkerSize',8)
              hold(app.UIAxes2,'on');
              x4=[1:300];
              y4=app.bEditField.Value*x4.^(app.aEditField.Value);
              plot(app.UIAxes2,x4,y4,'k-','Linewidth',2)
              axis(app.UIAxes2,[1 300 1 300])
              hold(app.UIAxes2,'off');
                end
        end
    end
    

    % Callbacks that handle component events
    methods (Access = private)

        % Code that executes after component creation
        function startupFcn(app, caller, namefile)
        

            if isempty(app.filename)==0
            % Read table array from file
            t = readtable(app.filename);

            Trials=t.Trials;
            Estimations=t.Estimations;
            Luminance_test=t.Luminance_test;
            logBr=log10(Estimations);
            logL=log10(Luminance_test);
            
            logL2=logL.^2;
            logLlogBr=logL.*logBr;

            T=table(Trials,Luminance_test,Estimations,logBr,logL,logL2,logLlogBr);
            

            app.UITable.Data = T;
            app.updateplot;
            end

        end

        % Button pushed function: LoaddatafileButton
        function LoaddatafileButtonPushed(app, event)
           
            mfile_name          = mfilename('fullpath');
            [pathstr,name,ext]  = fileparts(mfile_name);
            cd(pathstr);
            app.UIFigure.Visible = 'off';
            [filename,pathname] = uigetfile('*.xlsx');
            app.UIFigure.Visible = 'on';

            if isequal(filename,0) || isequal(pathname,0)
            disp('User pressed cancel')
            else
            disp(['User selected ', fullfile(pathname, filename)])
            app.filename= fullfile(pathname,filename);
            
            app.startupFcn;
             
            end
        end

        % Display data changed function: UITable
        function UITableDisplayDataChanged(app, event)
            %newDisplayData = app.UITable.DisplayData;
            app.updateplot;
        end

        % Cell selection callback: UITable
        function UITableCellSelection(app, event)
          app.Indices=event.Indices;
          app.updateplot;
        end

        % Button pushed function: SaveanalysisButton
        function SaveanalysisButtonPushed(app, event)
            t = app.UITable.DisplayData;  
         
            Trials=t.Trials;
            Estimations=t.Estimations;
            Luminance_test=t.Luminance_test;
            logBr=log10(Estimations);
            logL=log10(Luminance_test);
            
            logL2=logL.^2;
            logLlogBr=logL.*logBr;
            t = readtable(app.filename);
            Age=t.Age;
            Initials=t.Initials;
            T=table(Trials,Luminance_test,Estimations,logBr,logL,logL2,logLlogBr,Age,Initials);


            [pathstr,name,ext]  = fileparts(app.filename);
            name=[name '_copy' ext];
            filename= fullfile(pathstr,name);

            writetable(T,filename,'Sheet',1,'Range','A1');

            Stevens_exponent=app.aEditField.Value;
            Constant=app.bEditField.Value;
            T=table(Stevens_exponent,Constant);
            writetable(T,filename,'Sheet',1,'Range','K1');
        end
    end

    % Component initialization
    methods (Access = private)

        % Create UIFigure and components
        function createComponents(app)

            % Create UIFigure and hide until all components are created
            app.UIFigure = uifigure('Visible', 'off');
            app.UIFigure.Position = [100 100 831 563];
            app.UIFigure.Name = 'MATLAB App';

            % Create UIAxes
            app.UIAxes = uiaxes(app.UIFigure);
            title(app.UIAxes, 'Brightness estimation. Log-scale')
            xlabel(app.UIAxes, 'Relative Luminance (L)')
            ylabel(app.UIAxes, 'Brightness estimation (Br)')
            zlabel(app.UIAxes, 'Z')
            app.UIAxes.Box = 'on';
            app.UIAxes.XGrid = 'on';
            app.UIAxes.YGrid = 'on';
            app.UIAxes.Position = [446 305 360 199];

            % Create UIAxes2
            app.UIAxes2 = uiaxes(app.UIFigure);
            title(app.UIAxes2, 'Brightness estimation.  Linear-scale')
            xlabel(app.UIAxes2, 'Relative Luminance (L)')
            ylabel(app.UIAxes2, 'Brightness estimation (Br)')
            zlabel(app.UIAxes2, 'Z')
            app.UIAxes2.Box = 'on';
            app.UIAxes2.XGrid = 'on';
            app.UIAxes2.YGrid = 'on';
            app.UIAxes2.Position = [446 72 360 196];

            % Create AnalysisofbrightnessestimationLabel
            app.AnalysisofbrightnessestimationLabel = uilabel(app.UIFigure);
            app.AnalysisofbrightnessestimationLabel.FontSize = 14;
            app.AnalysisofbrightnessestimationLabel.FontWeight = 'bold';
            app.AnalysisofbrightnessestimationLabel.Position = [296 531 228 22];
            app.AnalysisofbrightnessestimationLabel.Text = 'Analysis of brightness estimation';

            % Create UITable
            app.UITable = uitable(app.UIFigure);
            app.UITable.ColumnName = {'Trials'; 'Luminance(L)'; 'Estimations(Br)*'; 'logBr'; 'logL'; 'logL^2'; 'logL*LogBr'};
            app.UITable.ColumnWidth = {'fit'};
            app.UITable.RowName = {};
            app.UITable.ColumnEditable = [false false true false false false false];
            app.UITable.CellSelectionCallback = createCallbackFcn(app, @UITableCellSelection, true);
            app.UITable.DisplayDataChangedFcn = createCallbackFcn(app, @UITableDisplayDataChanged, true);
            app.UITable.Position = [30 249 381 255];

            % Create LoaddatafileButton
            app.LoaddatafileButton = uibutton(app.UIFigure, 'push');
            app.LoaddatafileButton.ButtonPushedFcn = createCallbackFcn(app, @LoaddatafileButtonPushed, true);
            app.LoaddatafileButton.FontSize = 14;
            app.LoaddatafileButton.Position = [30 516 100 25];
            app.LoaddatafileButton.Text = 'Load data file';

            % Create SerranoPedrazaI2023Label
            app.SerranoPedrazaI2023Label = uilabel(app.UIFigure);
            app.SerranoPedrazaI2023Label.FontSize = 9;
            app.SerranoPedrazaI2023Label.Position = [9 8 110 22];
            app.SerranoPedrazaI2023Label.Text = 'Serrano-Pedraza, I. (2023)';

            % Create SaveanalysisButton
            app.SaveanalysisButton = uibutton(app.UIFigure, 'push');
            app.SaveanalysisButton.ButtonPushedFcn = createCallbackFcn(app, @SaveanalysisButtonPushed, true);
            app.SaveanalysisButton.Position = [129 216 130 23];
            app.SaveanalysisButton.Text = 'Save analysis';

            % Create nEditFieldLabel
            app.nEditFieldLabel = uilabel(app.UIFigure);
            app.nEditFieldLabel.HorizontalAlignment = 'right';
            app.nEditFieldLabel.FontAngle = 'italic';
            app.nEditFieldLabel.Position = [47 179 25 22];
            app.nEditFieldLabel.Text = 'n';

            % Create nEditField
            app.nEditField = uieditfield(app.UIFigure, 'numeric');
            app.nEditField.Editable = 'off';
            app.nEditField.HorizontalAlignment = 'center';
            app.nEditField.Position = [51 160 38 21];

            % Create SumlogLEditFieldLabel
            app.SumlogLEditFieldLabel = uilabel(app.UIFigure);
            app.SumlogLEditFieldLabel.HorizontalAlignment = 'right';
            app.SumlogLEditFieldLabel.Position = [106 180 59 22];
            app.SumlogLEditFieldLabel.Text = 'Sum(logL)';

            % Create SumlogLEditField
            app.SumlogLEditField = uieditfield(app.UIFigure, 'numeric');
            app.SumlogLEditField.Editable = 'off';
            app.SumlogLEditField.Position = [109 160 50 21];

            % Create SumlogBrEditFieldLabel
            app.SumlogBrEditFieldLabel = uilabel(app.UIFigure);
            app.SumlogBrEditFieldLabel.HorizontalAlignment = 'right';
            app.SumlogBrEditFieldLabel.Position = [170 180 65 22];
            app.SumlogBrEditFieldLabel.Text = 'Sum(logBr)';

            % Create SumlogBrEditField
            app.SumlogBrEditField = uieditfield(app.UIFigure, 'numeric');
            app.SumlogBrEditField.ValueDisplayFormat = '%.3f';
            app.SumlogBrEditField.Editable = 'off';
            app.SumlogBrEditField.Position = [181 160 50 21];

            % Create SumlogL2EditFieldLabel
            app.SumlogL2EditFieldLabel = uilabel(app.UIFigure);
            app.SumlogL2EditFieldLabel.HorizontalAlignment = 'right';
            app.SumlogL2EditFieldLabel.Position = [243 179 73 22];
            app.SumlogL2EditFieldLabel.Text = 'Sum(logL^2)';

            % Create SumlogL2EditField
            app.SumlogL2EditField = uieditfield(app.UIFigure, 'numeric');
            app.SumlogL2EditField.ValueDisplayFormat = '%.3f';
            app.SumlogL2EditField.Editable = 'off';
            app.SumlogL2EditField.Position = [243 159 78 21];

            % Create SumlogLlogBrEditFieldLabel
            app.SumlogLlogBrEditFieldLabel = uilabel(app.UIFigure);
            app.SumlogLlogBrEditFieldLabel.HorizontalAlignment = 'right';
            app.SumlogLlogBrEditFieldLabel.Position = [328 180 92 22];
            app.SumlogLlogBrEditFieldLabel.Text = 'Sum(logL*logBr)';

            % Create SumlogLlogBrEditField
            app.SumlogLlogBrEditField = uieditfield(app.UIFigure, 'numeric');
            app.SumlogLlogBrEditField.ValueDisplayFormat = '%.3f';
            app.SumlogLlogBrEditField.Editable = 'off';
            app.SumlogLlogBrEditField.Position = [333 158 79 21];

            % Create AEditFieldLabel
            app.AEditFieldLabel = uilabel(app.UIFigure);
            app.AEditFieldLabel.HorizontalAlignment = 'right';
            app.AEditFieldLabel.Position = [30 96 25 22];
            app.AEditFieldLabel.Text = 'A';

            % Create AEditField
            app.AEditField = uieditfield(app.UIFigure, 'numeric');
            app.AEditField.ValueDisplayFormat = '%.3f';
            app.AEditField.Editable = 'off';
            app.AEditField.Position = [63 96 50 21];

            % Create BEditFieldLabel
            app.BEditFieldLabel = uilabel(app.UIFigure);
            app.BEditFieldLabel.HorizontalAlignment = 'right';
            app.BEditFieldLabel.Position = [30 61 25 22];
            app.BEditFieldLabel.Text = 'B';

            % Create BEditField
            app.BEditField = uieditfield(app.UIFigure, 'numeric');
            app.BEditField.ValueDisplayFormat = '%.3f';
            app.BEditField.Editable = 'off';
            app.BEditField.Position = [63 61 50 21];

            % Create StevensparametersLabel
            app.StevensparametersLabel = uilabel(app.UIFigure);
            app.StevensparametersLabel.FontSize = 14;
            app.StevensparametersLabel.Position = [206 102 134 22];
            app.StevensparametersLabel.Text = 'Stevens'' parameters';

            % Create aEditFieldLabel
            app.aEditFieldLabel = uilabel(app.UIFigure);
            app.aEditFieldLabel.HorizontalAlignment = 'right';
            app.aEditFieldLabel.Position = [221 75 25 22];
            app.aEditFieldLabel.Text = 'a';

            % Create aEditField
            app.aEditField = uieditfield(app.UIFigure, 'numeric');
            app.aEditField.ValueDisplayFormat = '%.3f';
            app.aEditField.Editable = 'off';
            app.aEditField.Position = [254 75 50 21];

            % Create bEditFieldLabel
            app.bEditFieldLabel = uilabel(app.UIFigure);
            app.bEditFieldLabel.HorizontalAlignment = 'right';
            app.bEditFieldLabel.Position = [221 45 25 22];
            app.bEditFieldLabel.Text = 'b';

            % Create bEditField
            app.bEditField = uieditfield(app.UIFigure, 'numeric');
            app.bEditField.ValueDisplayFormat = '%.3f';
            app.bEditField.Editable = 'off';
            app.bEditField.Position = [254 45 50 21];

            % Create StevensexponentLabel
            app.StevensexponentLabel = uilabel(app.UIFigure);
            app.StevensexponentLabel.Position = [311 74 111 22];
            app.StevensexponentLabel.Text = '(Stevens'' exponent)';

            % Create LinearregressionLabel
            app.LinearregressionLabel = uilabel(app.UIFigure);
            app.LinearregressionLabel.FontSize = 14;
            app.LinearregressionLabel.Position = [30 123 112 22];
            app.LinearregressionLabel.Text = 'Linear regression';

            % Create EstimationscanbeeditedLabel
            app.EstimationscanbeeditedLabel = uilabel(app.UIFigure);
            app.EstimationscanbeeditedLabel.FontSize = 10;
            app.EstimationscanbeeditedLabel.Position = [288 228 124 22];
            app.EstimationscanbeeditedLabel.Text = '*Estimations can be edited';

            % Create StevensconstantLabel
            app.StevensconstantLabel = uilabel(app.UIFigure);
            app.StevensconstantLabel.Position = [314 45 108 22];
            app.StevensconstantLabel.Text = '(Stevens'' constant)';

            % Show the figure after all components are created
            app.UIFigure.Visible = 'on';
        end
    end

    % App creation and deletion
    methods (Access = public)

        % Construct app
        function app = AnalysisBrightness(varargin)

            % Create UIFigure and components
            createComponents(app)

            % Register the app with App Designer
            registerApp(app, app.UIFigure)

            % Execute the startup function
            runStartupFcn(app, @(app)startupFcn(app, varargin{:}))

            if nargout == 0
                clear app
            end
        end

        % Code that executes before app deletion
        function delete(app)

            % Delete UIFigure when app is deleted
            delete(app.UIFigure)
        end
    end
end