classdef CSF_Part_2 < matlab.apps.AppBase

    % Properties that correspond to app components
    properties (Access = public)
        UIFigure                        matlab.ui.Figure
        LoaddatafileButton              matlab.ui.control.Button
        TherowsLog10m0m0CS1m0canbeeditedLabel  matlab.ui.control.Label
        SerranoPedrazaI2023Label        matlab.ui.control.Label
        cm_3                            matlab.ui.control.Label
        cm_2                            matlab.ui.control.Label
        StimulussizeEditField           matlab.ui.control.NumericEditField
        StimulussizeEditFieldLabel      matlab.ui.control.Label
        DistancetothestimulusEditField  matlab.ui.control.NumericEditField
        DistancetothestimulusEditFieldLabel  matlab.ui.control.Label
        SpatialfrequencyincyclesperimageciLabel  matlab.ui.control.Label
        EstimationoftheContrastSensitivityFunction1DLabel  matlab.ui.control.Label
        ExampleButton                   matlab.ui.control.Button
        ResetButton                     matlab.ui.control.Button
        RemovefittingButton             matlab.ui.control.Button
        FitfunctionButton               matlab.ui.control.Button
        UITable                         matlab.ui.control.Table
        UIAxes                          matlab.ui.control.UIAxes
    end

    
    properties (Access = private)
        CallingApp; % Main app 

        B2=[0 1 1 2]';
        B4=[0 1 1 4]';
        B8=[0 1 1 8]';
        B16=[0 1 1 16]';
        B32=[0 1 1 32]';
        B64=[0 1 1 64]';
        B128=[0 1 1 128]';
        fitmodel=0;
    end
    
    properties (Access = public)
        filename; % Description 
        Indices;
    end
    
    methods (Access = private)
        
        
        function updateplot(app)

%%%%Dr. Ignacio Serrano-Pedraza 2023
%%%iserrano@ucm.es
%%%%%Contrast Sensitivity Function 1D (Campbell & Robson 1968)

%%%This is a demo for students to represent their data and to fit the
%%%contrast sensitivity function from Kelly (1975) adding three parameters (Peterson y Eckstein, 2012, eq. S9).
%%%CSF=(A*x.^beta).*exp(-a*x);

%%%Kelly,D.H. (1975).Spatial frequency selectivity in the retina,Vision Research,15,6,665-672,
%%%https://doi.org/10.1016/0042-6989(75)90282-5.
%%%Peterson MF, Eckstein MP. Looking just below the eyes is optimal across face recognition tasks. 
%%%Proc Natl Acad Sci U S A. 2012 Nov 27;109(48):E3314-23. doi: 10.1073/pnas.1214269109. Epub 2012 Nov 12.
%%%Campbell, F.W., & Robson, J.G. (1968). Application of Fourier analysis to the visibility of gratings.
%%%%Journal of Physiology, 197, 551- 566.


             app.UITable.RowName={'Log10(m0)'; 'm0'; 'CS=1/m0';'SF, c/deg'};
             t = app.UITable.DisplayData;
            app.B2=t.Var1;
            app.B4=t.Var2;
            app.B8=t.Var3;
            app.B16=t.Var4;
            app.B32=t.Var5;
            app.B64=t.Var6;
            app.B128=t.Var7;


            visualangle=2*atand((app.StimulussizeEditField.Value/2)/app.DistancetothestimulusEditField.Value);
            frequency=[2 4 8 16 32 64 128]./visualangle;

            data=[app.B2,app.B4,app.B8,app.B16,app.B32,app.B64,app.B128];
            data(4,:)=frequency;
            t=table(data(:,1),data(:,2),data(:,3),data(:,4),data(:,5),data(:,6),data(:,7));    
            app.UITable.Data = t;
            
        if isempty(app.Indices)==0
            File=unique(app.Indices(:,1));
            Column=unique(app.Indices(:,2));
            if File==1
            data(2,Column)=10.^data(File,Column);
            data(3,Column)=1./10.^data(File,Column);
            end
            if File==2
            data(1,Column)=log10(data(File,Column));
            data(3,Column)=1./10.^data(1,Column);
            end
            if File==3
            data(2,Column)=1./(data(File,Column));
            data(1,Column)=log10(data(2,Column));
            end
            data(4,:)=frequency;
            t=table(data(:,1),data(:,2),data(:,3),data(:,4),data(:,5),data(:,6),data(:,7));    
            app.UITable.Data = t;
        else
            data(2,:)=10.^data(1,:);
            data(3,:)=1./10.^data(1,:);
            t=table(data(:,1),data(:,2),data(:,3),data(:,4),data(:,5),data(:,6),data(:,7));    
            app.UITable.Data = t;
        end

        if app.fitmodel==1



        y=data(3,:);
        x=frequency;
        CSF=@(param,x) sum((param(1).*(x.^param(2)).*exp(-param(3).*x)-y).^2);
        param = fminsearch(@(param) CSF(param,x),[max(y);2.2;1.9],optimset('TolX',1E-10, 'TolFun', 1E-8,'MaxFunEvals',10000,'MaxIter',2000));

        %%Find freq with maximum sensitivity
        % xx=10.^linspace(log10(min(frequency)+0.2*min(frequency)),log10(max(frequency)-0.2*max(frequency)),10000);%%
        % y3=param(1).*(xx.^param(2)).*exp(-param(3).*xx);
        % CSF=@(MAX,param) abs(param(1).*(MAX.^param(2)).*exp(-param(3).*MAX)-max(y3));
        % MAX = fminsearch(@(MAX) CSF(MAX,param),[2],optimset('TolX',1E-10, 'TolFun', 1E-8,'MaxFunEvals',10000,'MaxIter',2000));
        
        MAX=param(2)/param(3);%%From Prof. Vicente Sierra-Vázquez taking the first derivative and equate it to zero.

        %%Find the spatial acuity
        CSF=@(AV,param) abs((param(1).*(AV.^param(2)).*exp(-param(3).*AV))-1)+10^4*any(AV < 4);
        AV = fminsearch(@(AV) CSF(AV,param),[20],optimset('TolX',1E-10, 'TolFun', 1E-8,'MaxFunEvals',10000,'MaxIter',2000));
        
        Ejex2=10.^linspace(log10(min(frequency)-0.4*min(frequency) ),log10(AV),100);%%
        y2=param(1).*(Ejex2.^param(2)).*exp(-param(3).*Ejex2);

       %%AV=exp(-(param(2)*lambertw(-(param(3)*exp(-(log(param(1))/param(2))/param(2))))+log(param(1)))/param(2));%%It doesn't work
       


        end

            axis(app.UIAxes,[min(frequency)-0.4*min(frequency) max(frequency)+0.4*max(frequency) 1 max(data(3,:))+0.8*max(data(3,:))])
            loglog(app.UIAxes,frequency,data(3,:),'ko','MarkerFaceColor','k','MarkerSize',9)
            hold(app.UIAxes,'on');
            if app.fitmodel==1
            loglog(app.UIAxes,Ejex2,y2,'k-','Linewidth',2)
            th_text=['Freq. Max. sensitivity = ',num2str(MAX,'%.2f'), ' c/deg'];
            text(app.UIAxes,frequency(1)+0.2*frequency(1),4, th_text, 'Color', 'k', 'FontSize',12)
            th_text=['Spatial acuity = ',num2str(AV,'%.2f'), ' c/deg'];
            text(app.UIAxes,frequency(1)+0.2*frequency(1),2, th_text, 'Color', 'k', 'FontSize',12)
             axis(app.UIAxes,[min(frequency)-0.4*min(frequency) AV+0.3*AV 1 max(data(3,:))+0.8*max(data(3,:))])
            end

           
           
            grid(app.UIAxes,'on');
            axis(app.UIAxes,'square')
            xlabel(app.UIAxes,'Spatial frequency, c/deg')
            ylabel(app.UIAxes,'Contrast Sensitivity (1/m0)')
            hold(app.UIAxes,'off');
        end
    end
    

    % Callbacks that handle component events
    methods (Access = private)

        % Code that executes after component creation
        function startupFcn(app, caller, namefile)
            
            if isempty(app.filename)==0
            % Read table array from file
            t = readtable(app.filename);
            Frequencies=t.Frequencies_c_i;
            for i=1:7
            Thresholds(i)=t.Thresholds(i);
            end

            Stimulus_size=t.Stimulus_size(1);
            Distance=t.Distance(1);
            if Stimulus_size>0 
            app.StimulussizeEditField.Value=Stimulus_size;
            end
            if Distance>0
            app.DistancetothestimulusEditField.Value=Distance;
            end
            t=table(Thresholds(1),Thresholds(2),Thresholds(3),Thresholds(4),Thresholds(5),Thresholds(6),Thresholds(7));     
            app.UITable.Data = t;
            app.Indices=[];
            app.fitmodel=0;
            app.updateplot;
            else

            app.filename=namefile;  
            % Read table array from file
            t = readtable(app.filename);
            Frequencies=t.Frequencies_c_i;
            for i=1:7
            Thresholds(i)=t.Thresholds(i);
            end
            Stimulus_size=t.Stimulus_size(1);
            Distance=t.Distance(1);
            if Stimulus_size>0 
            app.StimulussizeEditField.Value=Stimulus_size;
            end
            if Distance>0
            app.DistancetothestimulusEditField.Value=Distance;
            end

            t=table(Thresholds(1),Thresholds(2),Thresholds(3),Thresholds(4),Thresholds(5),Thresholds(6),Thresholds(7));     
            app.UITable.Data = t;
            app.Indices=[];
            app.fitmodel=0;
            app.updateplot;
            end
        end

        % Display data changed function: UITable
        function UITableDisplayDataChanged(app, event)
            %newDisplayData = app.UITable.DisplayData;


            app.updateplot;
        end

        % Cell selection callback: UITable
        function UITableCellSelection(app, event)
            
            app.Indices=event.Indices;
            app.updateplot;
        end

        % Value changed function: StimulussizeEditField
        function StimulussizeEditFieldValueChanged(app, event)
           %%% value = app.StimulussizeEditField.Value;
            app.updateplot;
        end

        % Value changed function: DistancetothestimulusEditField
        function DistancetothestimulusEditFieldValueChanged(app, event)
            %%%value = app.DistancetothestimulusEditField.Value;
            app.updateplot;
        end

        % Button pushed function: ResetButton
        function ResetButtonPushed(app, event)
        app.B2=[0 1 1 2]';
        app.B4=[0 1 1 4]';
        app.B8=[0 1 1 8]';
        app.B16=[0 1 1 16]';
        app.B32=[0 1 1 32]';
        app.B64=[0 1 1 64]';
        app.B128=[0 1 1 128]';
        t=table(app.B2,app.B4,app.B8,app.B16,app.B32,app.B64,app.B128);     
        app.UITable.Data = t;
        app.StimulussizeEditField.Value=60;
        app.DistancetothestimulusEditField.Value=300;
        app.fitmodel=0;
        app.updateplot;
        end

        % Button pushed function: FitfunctionButton
        function FitfunctionButtonPushed(app, event)
        app.fitmodel=1;
        app.updateplot;
        end

        % Button pushed function: RemovefittingButton
        function RemovefittingButtonPushed(app, event)
        app.fitmodel=0;
        app.updateplot;
        end

        % Button pushed function: ExampleButton
        function ExampleButtonPushed(app, event)
            
        app.B2=[-1.3 1 1 2]';
        app.B4=[-1.5 1 1 4]';
        app.B8=[-1.7 1 1 8]';
        app.B16=[-1.9 1 1 16]';
        app.B32=[-1.96 1 1 32]';
        app.B64=[-1.9 1 1 64]';
        app.B128=[-1.43 1 1 128]';  
        data=[app.B2,app.B4,app.B8,app.B16,app.B32,app.B64,app.B128];
        app.StimulussizeEditField.Value=108;
        app.DistancetothestimulusEditField.Value=697;
        visualangle=2*atand((app.StimulussizeEditField.Value/2)/app.DistancetothestimulusEditField.Value);
        frequency=[2 4 8 16 32 64 128]./visualangle;
        data(2,:)=10.^data(1,:);
        data(3,:)=1./10.^data(1,:);
        data(4,:)=frequency;
        t=table(data(:,1),data(:,2),data(:,3),data(:,4),data(:,5),data(:,6),data(:,7));    
        app.UITable.Data = t;
        app.fitmodel=0;
        app.updateplot;
        end

        % Button pushed function: LoaddatafileButton
        function LoaddatafileButtonPushed(app, event)
             mfile_name          = mfilename('fullpath');
            [pathstr,name,ext]  = fileparts(mfile_name);
            cd(pathstr);
            app.UIFigure.Visible = 'off';
            [filename,pathname] = uigetfile('*.xlsx');
            app.UIFigure.Visible = 'on';

            if isequal(filename,0) || isequal(pathname,0)
            disp('User pressed cancel')
            else
            disp(['User selected ', fullfile(pathname, filename)])
            app.filename= fullfile(pathname,filename);
            
            app.startupFcn;
            end
        end
    end

    % Component initialization
    methods (Access = private)

        % Create UIFigure and components
        function createComponents(app)

            % Create UIFigure and hide until all components are created
            app.UIFigure = uifigure('Visible', 'off');
            app.UIFigure.Position = [100 100 872 630];
            app.UIFigure.Name = 'MATLAB App';
            app.UIFigure.Resize = 'off';

            % Create UIAxes
            app.UIAxes = uiaxes(app.UIFigure);
            title(app.UIAxes, 'Contrast Sensitivity Function')
            zlabel(app.UIAxes, 'Z')
            app.UIAxes.Box = 'on';
            app.UIAxes.Position = [264 47 436 362];

            % Create UITable
            app.UITable = uitable(app.UIFigure);
            app.UITable.ColumnName = {'2'; '4'; '8'; '16'; '32'; '64'; '128'};
            app.UITable.RowName = {};
            app.UITable.ColumnEditable = true;
            app.UITable.CellSelectionCallback = createCallbackFcn(app, @UITableCellSelection, true);
            app.UITable.DisplayDataChangedFcn = createCallbackFcn(app, @UITableDisplayDataChanged, true);
            app.UITable.Position = [97 437 655 130];

            % Create FitfunctionButton
            app.FitfunctionButton = uibutton(app.UIFigure, 'push');
            app.FitfunctionButton.ButtonPushedFcn = createCallbackFcn(app, @FitfunctionButtonPushed, true);
            app.FitfunctionButton.FontWeight = 'bold';
            app.FitfunctionButton.Position = [689 327 100 23];
            app.FitfunctionButton.Text = 'Fit function';

            % Create RemovefittingButton
            app.RemovefittingButton = uibutton(app.UIFigure, 'push');
            app.RemovefittingButton.ButtonPushedFcn = createCallbackFcn(app, @RemovefittingButtonPushed, true);
            app.RemovefittingButton.FontWeight = 'bold';
            app.RemovefittingButton.Position = [689 266 100 23];
            app.RemovefittingButton.Text = 'Remove fitting';

            % Create ResetButton
            app.ResetButton = uibutton(app.UIFigure, 'push');
            app.ResetButton.ButtonPushedFcn = createCallbackFcn(app, @ResetButtonPushed, true);
            app.ResetButton.FontWeight = 'bold';
            app.ResetButton.Position = [689 198 100 23];
            app.ResetButton.Text = 'Reset';

            % Create ExampleButton
            app.ExampleButton = uibutton(app.UIFigure, 'push');
            app.ExampleButton.ButtonPushedFcn = createCallbackFcn(app, @ExampleButtonPushed, true);
            app.ExampleButton.Position = [105 235 100 23];
            app.ExampleButton.Text = 'Example';

            % Create EstimationoftheContrastSensitivityFunction1DLabel
            app.EstimationoftheContrastSensitivityFunction1DLabel = uilabel(app.UIFigure);
            app.EstimationoftheContrastSensitivityFunction1DLabel.FontSize = 14;
            app.EstimationoftheContrastSensitivityFunction1DLabel.FontWeight = 'bold';
            app.EstimationoftheContrastSensitivityFunction1DLabel.Position = [274 599 342 22];
            app.EstimationoftheContrastSensitivityFunction1DLabel.Text = 'Estimation of the Contrast Sensitivity Function 1D';

            % Create SpatialfrequencyincyclesperimageciLabel
            app.SpatialfrequencyincyclesperimageciLabel = uilabel(app.UIFigure);
            app.SpatialfrequencyincyclesperimageciLabel.Position = [329 566 230 22];
            app.SpatialfrequencyincyclesperimageciLabel.Text = 'Spatial frequency in cycles per image (c/i)';

            % Create DistancetothestimulusEditFieldLabel
            app.DistancetothestimulusEditFieldLabel = uilabel(app.UIFigure);
            app.DistancetothestimulusEditFieldLabel.HorizontalAlignment = 'right';
            app.DistancetothestimulusEditFieldLabel.Position = [16 377 134 22];
            app.DistancetothestimulusEditFieldLabel.Text = 'Distance to the stimulus';

            % Create DistancetothestimulusEditField
            app.DistancetothestimulusEditField = uieditfield(app.UIFigure, 'numeric');
            app.DistancetothestimulusEditField.Limits = [1 Inf];
            app.DistancetothestimulusEditField.ValueDisplayFormat = '%.2f';
            app.DistancetothestimulusEditField.ValueChangedFcn = createCallbackFcn(app, @DistancetothestimulusEditFieldValueChanged, true);
            app.DistancetothestimulusEditField.Position = [158 377 50 21];
            app.DistancetothestimulusEditField.Value = 300;

            % Create StimulussizeEditFieldLabel
            app.StimulussizeEditFieldLabel = uilabel(app.UIFigure);
            app.StimulussizeEditFieldLabel.HorizontalAlignment = 'right';
            app.StimulussizeEditFieldLabel.Position = [75 350 76 22];
            app.StimulussizeEditFieldLabel.Text = 'Stimulus size';

            % Create StimulussizeEditField
            app.StimulussizeEditField = uieditfield(app.UIFigure, 'numeric');
            app.StimulussizeEditField.Limits = [1 Inf];
            app.StimulussizeEditField.ValueDisplayFormat = '%.2f';
            app.StimulussizeEditField.ValueChangedFcn = createCallbackFcn(app, @StimulussizeEditFieldValueChanged, true);
            app.StimulussizeEditField.Position = [159 350 50 21];
            app.StimulussizeEditField.Value = 60;

            % Create cm_2
            app.cm_2 = uilabel(app.UIFigure);
            app.cm_2.HorizontalAlignment = 'right';
            app.cm_2.Position = [212 377 25 22];
            app.cm_2.Text = 'cm';

            % Create cm_3
            app.cm_3 = uilabel(app.UIFigure);
            app.cm_3.HorizontalAlignment = 'right';
            app.cm_3.Position = [212 349 25 22];
            app.cm_3.Text = 'cm';

            % Create SerranoPedrazaI2023Label
            app.SerranoPedrazaI2023Label = uilabel(app.UIFigure);
            app.SerranoPedrazaI2023Label.FontSize = 10;
            app.SerranoPedrazaI2023Label.Position = [17 16 122 22];
            app.SerranoPedrazaI2023Label.Text = 'Serrano-Pedraza, I. (2023)';

            % Create TherowsLog10m0m0CS1m0canbeeditedLabel
            app.TherowsLog10m0m0CS1m0canbeeditedLabel = uilabel(app.UIFigure);
            app.TherowsLog10m0m0CS1m0canbeeditedLabel.FontSize = 10;
            app.TherowsLog10m0m0CS1m0canbeeditedLabel.Position = [97 416 245 22];
            app.TherowsLog10m0m0CS1m0canbeeditedLabel.Text = '*The rows Log10(m0), m0, & CS=1/m0, can be edited';

            % Create LoaddatafileButton
            app.LoaddatafileButton = uibutton(app.UIFigure, 'push');
            app.LoaddatafileButton.ButtonPushedFcn = createCallbackFcn(app, @LoaddatafileButtonPushed, true);
            app.LoaddatafileButton.FontSize = 14;
            app.LoaddatafileButton.Position = [39 590 100 25];
            app.LoaddatafileButton.Text = 'Load data file';

            % Show the figure after all components are created
            app.UIFigure.Visible = 'on';
        end
    end

    % App creation and deletion
    methods (Access = public)

        % Construct app
        function app = CSF_Part_2(varargin)

            % Create UIFigure and components
            createComponents(app)

            % Register the app with App Designer
            registerApp(app, app.UIFigure)

            % Execute the startup function
            runStartupFcn(app, @(app)startupFcn(app, varargin{:}))

            if nargout == 0
                clear app
            end
        end

        % Code that executes before app deletion
        function delete(app)

            % Delete UIFigure when app is deleted
            delete(app.UIFigure)
        end
    end
end