classdef Colorimetry_light < matlab.apps.AppBase

    % Properties that correspond to app components
    properties (Access = public)
        UIFigure                        matlab.ui.Figure
        SerranoPedrazaI2023Label        matlab.ui.control.Label
        ColorimetryofthelightPanel      matlab.ui.container.Panel
        SimulationLabel                 matlab.ui.control.Label
        BEditField                      matlab.ui.control.NumericEditField
        BEditFieldLabel                 matlab.ui.control.Label
        GEditField                      matlab.ui.control.NumericEditField
        GEditFieldLabel                 matlab.ui.control.Label
        REditField                      matlab.ui.control.NumericEditField
        REditFieldLabel                 matlab.ui.control.Label
        Simulation                      matlab.ui.control.NumericEditField
        RybEditField                    matlab.ui.control.NumericEditField
        RybEditFieldLabel               matlab.ui.control.Label
        RrgEditField                    matlab.ui.control.NumericEditField
        RrgEditFieldLabel               matlab.ui.control.Label
        SaturationEditField             matlab.ui.control.EditField
        SaturationEditFieldLabel        matlab.ui.control.Label
        YellowEditField                 matlab.ui.control.EditField
        YellowEditFieldLabel            matlab.ui.control.Label
        RedEditField                    matlab.ui.control.EditField
        RedEditFieldLabel               matlab.ui.control.Label
        ChromaticresponseLHurvichLabel  matlab.ui.control.Label
        NormalizedkCheckBox             matlab.ui.control.CheckBox
        HueangleHuvEditField            matlab.ui.control.NumericEditField
        HueangleHuvEditFieldLabel       matlab.ui.control.Label
        SaturationSuvEditField          matlab.ui.control.NumericEditField
        SaturationSuvEditFieldLabel     matlab.ui.control.Label
        ChromaCuvEditField              matlab.ui.control.NumericEditField
        ChromaCuvEditFieldLabel         matlab.ui.control.Label
        LightnessLEditField             matlab.ui.control.NumericEditField
        LightnessLEditFieldLabel        matlab.ui.control.Label
        CIE1976LabLabel                 matlab.ui.control.Label
        bEditField                      matlab.ui.control.NumericEditField
        bEditFieldLabel                 matlab.ui.control.Label
        aEditField                      matlab.ui.control.NumericEditField
        aEditFieldLabel                 matlab.ui.control.Label
        vEditField_2                    matlab.ui.control.NumericEditField
        vEditField_2Label               matlab.ui.control.Label
        uEditField_2                    matlab.ui.control.NumericEditField
        uLabel                          matlab.ui.control.Label
        CIE1976LuvLabel                 matlab.ui.control.Label
        vEditField                      matlab.ui.control.NumericEditField
        vEditFieldLabel                 matlab.ui.control.Label
        uEditField                      matlab.ui.control.NumericEditField
        uEditFieldLabel                 matlab.ui.control.Label
        CIE1931Label                    matlab.ui.control.Label
        yEditField                      matlab.ui.control.NumericEditField
        yEditFieldLabel                 matlab.ui.control.Label
        xEditField                      matlab.ui.control.NumericEditField
        xEditFieldLabel                 matlab.ui.control.Label
        XEditField                      matlab.ui.control.NumericEditField
        ZEditField                      matlab.ui.control.NumericEditField
        ZEditFieldLabel                 matlab.ui.control.Label
        YEditField                      matlab.ui.control.NumericEditField
        YEditFieldLabel                 matlab.ui.control.Label
        TristimulusValuesCIE1931Label   matlab.ui.control.Label
        XEditFieldLabel                 matlab.ui.control.Label
        Label                           matlab.ui.control.Label
        CharacterizationoflightsLabel   matlab.ui.control.Label
        WavelengthEnergyLabel           matlab.ui.control.Label
        CurrentPositionEditField        matlab.ui.control.EditField
        ResetButton                     matlab.ui.control.Button
        UITable                         matlab.ui.control.Table
        UIAxes_3                        matlab.ui.control.UIAxes
        UIAxes_2                        matlab.ui.control.UIAxes
        UIAxes                          matlab.ui.control.UIAxes
    end


    properties (Access = private)

        wavelengths=[360:5:830]';
        Energy=0.*ones(95,1);

    end

    methods (Access = private)



        function updateplot(app)

            %%%Dr. Ignacio Serrano-Pedraza (2023)

            %%NOTE: The idea of using an image to put under the chromaticity diagram was
            %%taken from Prashant Patil (2020, Matlab software). This allows the
            %%program to run faster.
            %%The  color images that I am using are programmed in PostScript (.eps) using the libraries from
            %%Gernot Hoffmann but changing the code to create sRGB gamut where the
            %%white point corresponds to the illuminant D65.

            %References:

            %%Wyszecki & Stiles (1982) Color Science: Concepts and Methods,
            %%Quantitative Data and Formulae. Second Edition.

            %%%Hurvich, L. (1982) Color Vision.
            load('CIE1931_full.mat')%%Color matching Functions. Reference: http://cvrl.ioo.ucl.ac.uk/cmfs.htm
            lambda=data.lambda;
            [h p]=find(lambda'==app.wavelengths);
            lambda=data.lambda(p);
            d65=data.d65(p)';%%%from http://www.npsg.uwaterloo.ca/data/illuminant/d65.m
            x=data.xFcn(p);
            y=data.yFcn(p);
            z=data.zFcn(p);




            % Get Table UI component data
            t = app.UITable.DisplayData;
            app.wavelengths=t.Var1;
            app.Energy=t.Var2;
            T=table(app.wavelengths,app.Energy);
            app.UITable.Data = T;

            plot(app.UIAxes_3,app.wavelengths,app.Energy,'ko-','Linewidth',2,'HitTest','off')

            xlabel(app.UIAxes_3,'Wavelength (nm)')
            ylabel(app.UIAxes_3,'Relative Energy')
            title(app.UIAxes_3,'Energy distribution of the light')
            axis(app.UIAxes_3,[360 830 0 5+max(app.Energy)+0.1*max(app.Energy)])
            grid(app.UIAxes_3,'on');

            if app.NormalizedkCheckBox.Value==0
                k=1;
            else

                k=100./sum(y.*app.Energy);
            end
            if k==Inf;k=0;end
            %k=1;%100./sum(y.*d65);%%Normalization Factor Wiszecki & Stiles pp. 157
            XA=k*sum(x.*app.Energy);
            YA=k*sum(y.*app.Energy);
            ZA=k*sum(z.*app.Energy);

            app.XEditField.Value=XA;
            app.YEditField.Value=YA;
            app.ZEditField.Value=ZA;

            x_coord_A=XA/(XA+YA+ZA);%%x
            y_coord_A=YA/(XA+YA+ZA);%%y
            z_coord_A=ZA/(XA+YA+ZA);%%z

            denominator=(XA+YA+ZA);

            if denominator==0;x_coord_A=0;y_coord_A=0;end
            app.xEditField.Value=x_coord_A;
            app.yEditField.Value=y_coord_A;

            u_coord_A=4*XA/(XA+15*YA+3*ZA);%%u'
            v_coord_A=9*YA/(XA+15*YA+3*ZA);%%v'
            if denominator==0;u_coord_A=0;v_coord_A=0;end
            app.uEditField.Value=u_coord_A;
            app.vEditField.Value=v_coord_A;



            Xn=100;
            Yn=100;
            Zn=100;
            u_coord_n=4*Xn/(Xn+15*Yn+3*Zn);%%%u'
            v_coord_n=9*Yn/(Xn+15*Yn+3*Zn);%%%v'
            if (YA/Yn)<=0.008856;lightnessA=903.3*(YA/Yn);else;lightnessA=(116*(YA/Yn)^(1/3))-16;end
            CIE_uA=13*lightnessA*(u_coord_A-u_coord_n);%%u*
            CIE_vA=13*lightnessA*(v_coord_A-v_coord_n);%%v*
            if denominator==0;CIE_uA=0;CIE_vA=0;end
            app.uEditField_2.Value=CIE_uA;
            app.vEditField_2.Value=CIE_vA;


            CIE_aA=500*((XA/Xn)^(1/3)-(YA/Yn)^(1/3));
            CIE_bA=200*((YA/Yn)^(1/3)-(ZA/Zn)^(1/3));
            if denominator==0;CIE_aA=0;CIE_bA=0;end
            app.aEditField.Value=CIE_aA;
            app.bEditField.Value=CIE_bA;

            Chroma_uvA=(CIE_uA^2+CIE_vA^2)^(1/2);
            Saturation_uvA=Chroma_uvA/lightnessA;
            Hueangle_uvA=atand(CIE_vA/CIE_uA);
            if (CIE_uA>0 && CIE_vA>0);Hueangle_uvA=Hueangle_uvA;end
            if (CIE_uA<0 && CIE_vA>0);Hueangle_uvA=Hueangle_uvA+180;end
            if (CIE_uA<0 && CIE_vA<0);Hueangle_uvA=Hueangle_uvA+180;end
            if (CIE_uA>0 && CIE_vA<0);Hueangle_uvA=Hueangle_uvA+360;end
            if isnan(Hueangle_uvA)==1;Hueangle_uvA=0;end

            if denominator==0;lightnessA;Chroma_uvA=0;Saturation_uvA=0;Hueangle_uvA=0;end

            app.LightnessLEditField.Value=lightnessA;
            app.ChromaCuvEditField.Value=Chroma_uvA;
            app.SaturationSuvEditField.Value=Saturation_uvA;
            app.HueangleHuvEditField.Value=Hueangle_uvA;



            % %%%%%%%%%%CHROMATIC RESPONSE (Leo Hurvich)%%%%%%%%%%%
            %
            R_rg_A=XA-YA;
            R_yb_A=0.4*YA-0.4*ZA;

            
            C_rg_A=100*abs(R_rg_A)/(abs(R_rg_A)+abs(R_yb_A));
            C_yb_A=100*abs(R_yb_A)/(abs(R_rg_A)+abs(R_yb_A));

            saturationhurvich_A=(abs(R_rg_A)+abs(R_yb_A))/((abs(R_rg_A)+abs(R_yb_A)+YA));%%%Leo Hurvich pp. 79-82


if denominator==0;C_rg_A=0;C_yb_A=0;saturationhurvich_A=0;R_rg_A=0;R_yb_A=0;end
h_text=[num2str(C_rg_A,'%3.2f'), ' %'];
app.RedEditField.Value=h_text;

h_text=[num2str(C_yb_A,'%3.2f'), ' %'];
app.YellowEditField.Value=h_text;

h_text=[num2str(saturationhurvich_A,'%3.3f')];
app.SaturationEditField.Value=h_text;


app.RrgEditField.Value=R_rg_A;
app.RybEditField.Value=R_yb_A;

if R_rg_A>=0
    app.RedEditFieldLabel.BackgroundColor = [1 0 0];
    app.RedEditFieldLabel.Text = 'Red';
    app.RedEditFieldLabel.FontColor = [1 1 1];

else
    app.RedEditFieldLabel.BackgroundColor = [0 1 0];
    app.RedEditFieldLabel.Text = 'Green';
    app.RedEditFieldLabel.FontColor = [0 0 0];

end
if R_yb_A>0
    app.YellowEditFieldLabel.BackgroundColor = [1 1 0];
    app.YellowEditFieldLabel.Text = 'Yellow';
    app.YellowEditFieldLabel.FontColor = [0 0 0];

else
    app.YellowEditFieldLabel.BackgroundColor = [0 0 1];
    app.YellowEditFieldLabel.Text = 'Blue';
    app.YellowEditFieldLabel.FontColor = [1 1 1];

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %%White point E http://www.brucelindbloom.com/index.html?Eqn_XYZ_to_RGB.html
    Rm=[2.3706743 -0.9000405 -0.4706338];
    Gm=[-0.5138850  1.4253036  0.0885814];
    Bm=[0.0052982 -0.0146949  1.0093968];
    %
    X=XA./max([XA YA ZA]);
    Y=YA./max([XA YA ZA]);
    Z=ZA./max([XA YA ZA]);
    Rx=Rm(1)*X+Rm(2)*Y+Rm(3)*Z;
    Gy=Gm(1)*X+Gm(2)*Y+Gm(3)*Z;
    Bz=Bm(1)*X+Bm(2)*Y+Bm(3)*Z;
    
    if Rx>=0.0031308;R=Rx^(1/2.2);else;R=0;end
    if Gy>=0.0031308;G=Gy^(1/2.2);else;G=0;end
    if Bz>=0.0031308;B=Bz^(1/2.2);else;B=0;end
    
    if R>1;R=1;end
    if G>1;G=1;end
    if B>1;B=1;end
    
    if denominator==0;R=0;G=0;B=0;end
    %app.Simulation.AllowEmpty='On';
    %app.Simulation.Value=[];
    app.Simulation.BackgroundColor = [R G B];
    app.Simulation.FontColor = [R G B];

    app.REditField.Value=round(R*255);
    app.GEditField.Value=round(G*255);
    app.BEditField.Value=round(B*255);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%Hurvich, L. (1982) Color Vision.
            load('CIE1931_full.mat')%%Color matching Functions. Reference: http://cvrl.ioo.ucl.ac.uk/cmfs.htm
            lambda=data.lambda;
            x=data.xFcn;
            y=data.yFcn;
            z=data.zFcn;
            d65=data.d65;%%%from http://www.npsg.uwaterloo.ca/data/illuminant/d65.m
            c_x=x./(x+y+z);
            c_y=y./(x+y+z);
            image_color=imread('CIE1931_E','jpg');
            tam=size(image_color);
            MAYOR=tam(2);
            MENOR=1;
            MAX1=max(c_x);
            MIN1=min(c_x);
            ax=(MAYOR - MENOR)/(MAX1-MIN1);
            bx=MENOR-ax*MIN1;
            C_x = ax*c_x+bx;

            MAYOR=1;
            MENOR=tam(1);
            MAX1=max(c_y);
            MIN1=min(c_y);
            ay=(MAYOR - MENOR)/(MAX1-MIN1);
            by=MENOR-ay*MIN1;
            C_y = ay*c_y+by;

            X_whitepoint=ax*0.333+bx;
            Y_whitepoint=ay*0.333+by;

            gridx=ax*[0:0.1:0.9]+bx;
            gridy=ay*[0:0.1:0.9]+by;

            image(app.UIAxes,image_color)
            hold(app.UIAxes,'on');

            for i=1:9
                plot(app.UIAxes,[gridx(i) gridx(i)],[min(gridy) max(gridy)],'k-','linewidth',0.5,'Color',[0.7 0.7 0.7])
                hold(app.UIAxes,'on');
                plot(app.UIAxes,[min(gridx) max(gridx)],[gridy(i) gridy(i)],'k-','linewidth',0.5,'Color',[0.7 0.7 0.7])
            end
            hold(app.UIAxes,'on');
            plot(app.UIAxes,C_x,C_y,'k-',[C_x(1) C_x(end)],[C_y(1) C_y(end)],'k-','linewidth',1.2)
            hold(app.UIAxes,'on');
            plot(app.UIAxes,(C_x([41 111:10:end-210 341])),C_y([41 111:10:end-210 341]),'ko','MarkerFaceColor','w')
            hold(app.UIAxes,'on');
            plot(app.UIAxes,X_whitepoint,Y_whitepoint,'ko','MarkerFaceColor','w','MarkerSize',8)
            hold(app.UIAxes,'on');

            %%%AQUI VA LA LUZ
            X_CC=ax*x_coord_A+bx;
            Y_CC=ay*y_coord_A+by;

            plot(app.UIAxes,X_CC,Y_CC,'ko','MarkerFaceColor','k','MarkerSize',8)
            hold(app.UIAxes,'on');

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            CC_x = ax*0.7+bx;
            CC_y = ay*0.85+by;
            plot(app.UIAxes,CC_x,CC_y,'ko','MarkerFaceColor','w','MarkerSize',8)
            hold(app.UIAxes,'on');
            text(app.UIAxes,CC_x+27,CC_y, 'E', 'Color', 'k', 'FontSize',15/1.3)
            CC_x = ax*0.7+bx;
            CC_y = ay*0.78+by;
            plot(app.UIAxes,CC_x,CC_y,'ko','MarkerFaceColor','k','MarkerSize',8)
            text(app.UIAxes,CC_x+27,CC_y, 'Light', 'Color', 'k', 'FontSize',15/1.3)
            hold(app.UIAxes,'on');


            axis(app.UIAxes,[ax*0+bx ax*0.9+bx ay*0.9+by ay*0+by])
            labels=linspace(ax*0+bx,ax*0.9+bx,10);
            T = linspace(0,0.9,10);
            str=num2str(T','%10.2f');
            set(app.UIAxes,'xTick',labels,'xTickLabel',{str})
            labels=linspace(ay*0.9+by,ay*0+by,10);
            T = linspace(0.9,0,10);
            str=num2str(T','%10.2f');
            set(app.UIAxes,'yTick',labels,'yTickLabel',{str})
            lambdas=lambda([41 111:10:end-210 341]);
            c=[41 111:10:length(lambda)-210 341];
            for i=1:length(lambdas)
                th_text=[num2str(lambdas(i))];
                if lambdas(i)<=510
                    text(app.UIAxes,C_x(c(i))-95,C_y(c(i))-20, th_text, 'Color', 'k', 'FontSize',15/1.3)
                else
                    text(app.UIAxes,C_x(c(i))+20,C_y(c(i))-10, th_text, 'Color', 'k', 'FontSize',15/1.3)
                end
            end
            hold(app.UIAxes,'off');
            % grid(app.UIAxes,'off');
            axis(app.UIAxes,'square')
            xlabel(app.UIAxes,'x')
            ylabel(app.UIAxes,'y')
            title(app.UIAxes,'Chromaticity diagram CIE 1931')
            hold(app.UIAxes,'off');

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            image_color=imread('CIE1976_E','jpg');
            tam=size(image_color);

            u_x=4*c_x./(-2*c_x+12*c_y+3);
            v_y=9*c_y./(-2*c_x+12*c_y+3);

            MAYOR=tam(2);
            MENOR=1;
            MAX1=max(u_x);
            MIN1=min(u_x);
            au=(MAYOR - MENOR)/(MAX1-MIN1);
            bu=MENOR-au*MIN1;
            U_x = au*u_x+bu;

            MAYOR=1;
            MENOR=tam(1);
            MAX1=max(v_y);
            MIN1=min(v_y);
            av=(MAYOR - MENOR)/(MAX1-MIN1);
            bv=MENOR-av*MIN1;
            V_y = av*v_y+bv;

            u_whitepoint=4*0.333./(-2*0.333+12*0.333+3);
            v_whitepoint=9*0.333./(-2*0.333+12*0.333+3);

            U_whitepoint= au*u_whitepoint+bu;
            V_whitepoint =av*v_whitepoint+bv;

            gridx=au*[0:0.1:0.7]+bu;
            gridy=av*[0:0.1:0.7]+bv;

            image(app.UIAxes_2,image_color)
            hold(app.UIAxes_2,'on');

            for i=1:8
                plot(app.UIAxes_2,[gridx(i) gridx(i)],[min(gridy) max(gridy)],'k-','linewidth',0.5,'Color',[0.7 0.7 0.7])
                hold(app.UIAxes_2,'on');
                plot(app.UIAxes_2,[min(gridx) max(gridx)],[gridy(i) gridy(i)],'k-','linewidth',0.5,'Color',[0.7 0.7 0.7])
            end
            hold(app.UIAxes_2,'on');
            plot(app.UIAxes_2,U_x,V_y,'k-',[U_x(1) U_x(end)],[V_y(1) V_y(end)],'k-','linewidth',1.2)
            hold(app.UIAxes_2,'on');
            plot(app.UIAxes_2,(U_x([41 111 131 101:20:end-210 341])),V_y([41 111 131 101:20:end-210 341]),'ko','MarkerFaceColor','w')
            hold(app.UIAxes_2,'on');
            plot(app.UIAxes_2,U_whitepoint,V_whitepoint,'ko','MarkerFaceColor','w','MarkerSize',8)
            hold(app.UIAxes_2,'on');

            %%%AQUI VA LA LUZ
            U_CC= au*u_coord_A+bu;
            V_CC =av*v_coord_A+bv;
            plot(app.UIAxes_2,U_CC,V_CC,'ko','MarkerFaceColor','k','MarkerSize',8)
            hold(app.UIAxes_2,'on');
            %%%%%LEGEND%%%%%%%%%%%%%%%%%%
            UU_u = au*0.5+bu;
            VV_v = av*0.2+bv;

            plot(app.UIAxes_2,UU_u ,VV_v ,'ko','MarkerFaceColor','w','MarkerSize',8)
            hold(app.UIAxes_2,'on');
            text(app.UIAxes_2,UU_u+24,VV_v, 'E', 'Color', 'k', 'FontSize',15/1.3)
            UU_u = au*0.5+bu;
            VV_v = av*0.15+bv;
            plot(app.UIAxes_2,UU_u,VV_v,'ko','MarkerFaceColor','k','MarkerSize',8)
            text(app.UIAxes_2,UU_u+24,VV_v, 'Light', 'Color', 'k', 'FontSize',15/1.3)
            hold(app.UIAxes_2,'on');


            axis(app.UIAxes_2,[au*0+bu au*0.7+bu av*0.7+bv av*0+bv])
            labels=linspace(au*0+bu,au*0.7+bu,8);
            T = linspace(0,0.7,8);
            str=num2str(T','%10.2f');
            set(app.UIAxes_2,'xTick',labels,'xTickLabel',{str})
            labels=linspace(av*0.7+bv,av*0+bv,8);
            T = linspace(0.7,0,8);
            str=num2str(T','%10.2f');
            set(app.UIAxes_2,'yTick',labels,'yTickLabel',{str})
            lambdas=lambda([41 111 131 101:20:end-210 341]);
            c=[41 111 131 101:20:length(lambda)-210 341];
            for i=1:length(lambdas)
                th_text=[num2str(lambdas(i))];
                if lambdas(i)<=510
                    text(app.UIAxes_2,(U_x(c(i))-75),V_y(c(i))-15, th_text, 'Color', 'k', 'FontSize',15/1.3)
                else
                    text(app.UIAxes_2,(U_x(c(i))+15),V_y(c(i))-17, th_text, 'Color', 'k', 'FontSize',15/1.3)
                end
            end
            % grid(app.UIAxes_2,'off');
            axis(app.UIAxes_2,'square')
            xlabel(app.UIAxes_2,'u''')
            ylabel(app.UIAxes_2,'v''')
            title(app.UIAxes_2,'Chromaticity diagram CIE 1976')
            hold(app.UIAxes_2,'off');



        end
    end


    % Callbacks that handle component events
    methods (Access = private)

        % Code that executes after component creation
        function startupFcn(app)
            % Get Table UI component data

            %%%%%%%%%%%%%%%%%%%%%% This code was adapted from Adam Danz 18-3-2021
            %%%https://es.mathworks.com/matlabcentral/answers/775147-how-to-create-mouse-movement-
            %%%event-on-uiaxes-in-app-designer-to-catch-cursor-location-on-the-axes#answer_651357

            pm.enterFcn = [];
            pm.exitFcn  = @(~,~) set(app.CurrentPositionEditField, 'Value', '');
            pm.traverseFcn = @(~,~) set(app.CurrentPositionEditField, 'Value',...
                sprintf('%.0f,  %.0f',[round(app.UIAxes_3.CurrentPoint(2,1)/5)*5  round(app.UIAxes_3.CurrentPoint(2,2))]));
            iptSetPointerBehavior(app.UIAxes_3, pm)
            iptPointerManager(app.UIFigure,'enable');
            set(app.UIFigure,'WindowButtonMotionFcn',@(~,~)NaN) %dummy fcn so that currentpoint is continually updated
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            T=table(app.wavelengths,app.Energy);
            app.UITable.Data = T;
            %app.Simulation.AllowEmpty='On';
            %app.Simulation.Value=[];
            app.updateplot;
        end

        % Display data changed function: UITable
        function UITableDisplayDataChanged(app, event)

            app.updateplot;
        end

        % Button down function: UIAxes_3
        function UIAxes_3ButtonDown(app, event)


            x=round(app.UIAxes_3.CurrentPoint(2,1));
            y=round(app.UIAxes_3.CurrentPoint(2,2));
            [h p]=min(abs(x-app.wavelengths));
            if y<0;y=0;end
            app.Energy(p)=y;

            T=table(app.wavelengths,app.Energy);
            app.UITable.Data = T;


            app.updateplot;
        end

        % Button pushed function: ResetButton
        function ResetButtonPushed(app, event)
            app.wavelengths=[360:5:830]';
            app.Energy=0.*ones(95,1);
            T=table(app.wavelengths,app.Energy);
            app.UITable.Data = T;
            app.updateplot;
        end

        % Value changed function: NormalizedkCheckBox
        function NormalizedkCheckBoxValueChanged(app, event)
            value = app.NormalizedkCheckBox.Value;
            app.updateplot;
        end
    end

    % Component initialization
    methods (Access = private)

        % Create UIFigure and components
        function createComponents(app)

            % Create UIFigure and hide until all components are created
            app.UIFigure = uifigure('Visible', 'off');
            app.UIFigure.AutoResizeChildren = 'off';
            app.UIFigure.Position = [100 100 1167 691];
            app.UIFigure.Name = 'MATLAB App';
            app.UIFigure.Resize = 'off';
            app.UIFigure.Scrollable = 'on';

            % Create UIAxes
            app.UIAxes = uiaxes(app.UIFigure);
            title(app.UIAxes, 'Title')
            xlabel(app.UIAxes, 'X')
            ylabel(app.UIAxes, 'Y')
            zlabel(app.UIAxes, 'Z')
            app.UIAxes.Toolbar.Visible = 'off';
            app.UIAxes.DataAspectRatio = [2 2 2];
            app.UIAxes.PlotBoxAspectRatio = [1 1 1];
            app.UIAxes.Box = 'on';
            app.UIAxes.Clipping = 'off';
            app.UIAxes.Position = [428 4 365 353];

            % Create UIAxes_2
            app.UIAxes_2 = uiaxes(app.UIFigure);
            title(app.UIAxes_2, 'Title')
            xlabel(app.UIAxes_2, 'X')
            ylabel(app.UIAxes_2, 'Y')
            zlabel(app.UIAxes_2, 'Z')
            app.UIAxes_2.Toolbar.Visible = 'off';
            app.UIAxes_2.DataAspectRatio = [2 2 2];
            app.UIAxes_2.PlotBoxAspectRatio = [1 1 1];
            app.UIAxes_2.Box = 'on';
            app.UIAxes_2.Clipping = 'off';
            app.UIAxes_2.Position = [803 4 355 353];

            % Create UIAxes_3
            app.UIAxes_3 = uiaxes(app.UIFigure);
            title(app.UIAxes_3, 'Title')
            xlabel(app.UIAxes_3, 'X')
            ylabel(app.UIAxes_3, 'Y')
            zlabel(app.UIAxes_3, 'Z')
            app.UIAxes_3.Toolbar.Visible = 'off';
            app.UIAxes_3.ClippingStyle = 'rectangle';
            app.UIAxes_3.Box = 'on';
            app.UIAxes_3.Clipping = 'off';
            app.UIAxes_3.ButtonDownFcn = createCallbackFcn(app, @UIAxes_3ButtonDown, true);
            app.UIAxes_3.Position = [11 65 322 190];

            % Create UITable
            app.UITable = uitable(app.UIFigure);
            app.UITable.ColumnName = {'Wavelength (nm)'; 'Energy'};
            app.UITable.RowName = {};
            app.UITable.ColumnEditable = [false true];
            app.UITable.DisplayDataChangedFcn = createCallbackFcn(app, @UITableDisplayDataChanged, true);
            app.UITable.Position = [36 314 229 303];

            % Create ResetButton
            app.ResetButton = uibutton(app.UIFigure, 'push');
            app.ResetButton.ButtonPushedFcn = createCallbackFcn(app, @ResetButtonPushed, true);
            app.ResetButton.FontWeight = 'bold';
            app.ResetButton.Position = [129 13 111 26];
            app.ResetButton.Text = 'Reset';

            % Create CurrentPositionEditField
            app.CurrentPositionEditField = uieditfield(app.UIFigure, 'text');
            app.CurrentPositionEditField.HorizontalAlignment = 'center';
            app.CurrentPositionEditField.FontSize = 14;
            app.CurrentPositionEditField.Position = [129 259 111 22];

            % Create WavelengthEnergyLabel
            app.WavelengthEnergyLabel = uilabel(app.UIFigure);
            app.WavelengthEnergyLabel.Position = [129 284 111 22];
            app.WavelengthEnergyLabel.Text = 'Wavelength, Energy';

            % Create CharacterizationoflightsLabel
            app.CharacterizationoflightsLabel = uilabel(app.UIFigure);
            app.CharacterizationoflightsLabel.FontSize = 14;
            app.CharacterizationoflightsLabel.FontWeight = 'bold';
            app.CharacterizationoflightsLabel.Position = [620 667 173 22];
            app.CharacterizationoflightsLabel.Text = 'Characterization of lights';

            % Create Label
            app.Label = uilabel(app.UIFigure);
            app.Label.Interruptible = 'off';
            app.Label.HorizontalAlignment = 'center';
            app.Label.WordWrap = 'on';
            app.Label.Position = [11 616 280 73];
            app.Label.Text = 'Click the mouse on top of the light plot or edit the table to select the relative energy';

            % Create ColorimetryofthelightPanel
            app.ColorimetryofthelightPanel = uipanel(app.UIFigure);
            app.ColorimetryofthelightPanel.AutoResizeChildren = 'off';
            app.ColorimetryofthelightPanel.Title = 'Colorimetry of the light';
            app.ColorimetryofthelightPanel.Position = [319 378 818 276];

            % Create XEditFieldLabel
            app.XEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.XEditFieldLabel.HorizontalAlignment = 'center';
            app.XEditFieldLabel.Position = [72 207 25 22];
            app.XEditFieldLabel.Text = 'X';

            % Create TristimulusValuesCIE1931Label
            app.TristimulusValuesCIE1931Label = uilabel(app.ColorimetryofthelightPanel);
            app.TristimulusValuesCIE1931Label.FontSize = 11;
            app.TristimulusValuesCIE1931Label.Position = [104 230 141 22];
            app.TristimulusValuesCIE1931Label.Text = 'Tristimulus Values CIE 1931';

            % Create YEditFieldLabel
            app.YEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.YEditFieldLabel.HorizontalAlignment = 'center';
            app.YEditFieldLabel.Position = [155 207 25 22];
            app.YEditFieldLabel.Text = 'Y';

            % Create YEditField
            app.YEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.YEditField.ValueDisplayFormat = '%.2f';
            app.YEditField.Editable = 'off';
            app.YEditField.HorizontalAlignment = 'center';
            app.YEditField.Position = [134 189 68 21];

            % Create ZEditFieldLabel
            app.ZEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.ZEditFieldLabel.HorizontalAlignment = 'center';
            app.ZEditFieldLabel.Position = [238 207 25 22];
            app.ZEditFieldLabel.Text = 'Z';

            % Create ZEditField
            app.ZEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.ZEditField.ValueDisplayFormat = '%.2f';
            app.ZEditField.Editable = 'off';
            app.ZEditField.HorizontalAlignment = 'center';
            app.ZEditField.Position = [217 189 68 21];

            % Create XEditField
            app.XEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.XEditField.ValueDisplayFormat = '%.2f';
            app.XEditField.Editable = 'off';
            app.XEditField.HorizontalAlignment = 'center';
            app.XEditField.Position = [51 189 68 21];

            % Create xEditFieldLabel
            app.xEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.xEditFieldLabel.HorizontalAlignment = 'center';
            app.xEditFieldLabel.Position = [54 66 25 22];
            app.xEditFieldLabel.Text = 'x';

            % Create xEditField
            app.xEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.xEditField.ValueDisplayFormat = '%.3f';
            app.xEditField.Editable = 'off';
            app.xEditField.HorizontalAlignment = 'center';
            app.xEditField.Position = [79 66 68 21];

            % Create yEditFieldLabel
            app.yEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.yEditFieldLabel.HorizontalAlignment = 'center';
            app.yEditFieldLabel.Position = [54 35 25 22];
            app.yEditFieldLabel.Text = 'y';

            % Create yEditField
            app.yEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.yEditField.ValueDisplayFormat = '%.3f';
            app.yEditField.Editable = 'off';
            app.yEditField.HorizontalAlignment = 'center';
            app.yEditField.Position = [79 35 68 21];

            % Create CIE1931Label
            app.CIE1931Label = uilabel(app.ColorimetryofthelightPanel);
            app.CIE1931Label.Position = [84 95 54 22];
            app.CIE1931Label.Text = 'CIE 1931';

            % Create uEditFieldLabel
            app.uEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.uEditFieldLabel.HorizontalAlignment = 'center';
            app.uEditFieldLabel.Position = [194 65 25 22];
            app.uEditFieldLabel.Text = 'u''';

            % Create uEditField
            app.uEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.uEditField.ValueDisplayFormat = '%.3f';
            app.uEditField.Editable = 'off';
            app.uEditField.HorizontalAlignment = 'center';
            app.uEditField.Position = [219 65 68 21];

            % Create vEditFieldLabel
            app.vEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.vEditFieldLabel.HorizontalAlignment = 'center';
            app.vEditFieldLabel.Position = [194 34 25 22];
            app.vEditFieldLabel.Text = 'v''';

            % Create vEditField
            app.vEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.vEditField.ValueDisplayFormat = '%.3f';
            app.vEditField.Editable = 'off';
            app.vEditField.HorizontalAlignment = 'center';
            app.vEditField.Position = [219 34 68 21];

            % Create CIE1976LuvLabel
            app.CIE1976LuvLabel = uilabel(app.ColorimetryofthelightPanel);
            app.CIE1976LuvLabel.Position = [252 95 102 22];
            app.CIE1976LuvLabel.Text = 'CIE 1976 (L* u* v*)';

            % Create uLabel
            app.uLabel = uilabel(app.ColorimetryofthelightPanel);
            app.uLabel.HorizontalAlignment = 'center';
            app.uLabel.Position = [298 64 25 22];
            app.uLabel.Text = 'u*';

            % Create uEditField_2
            app.uEditField_2 = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.uEditField_2.ValueDisplayFormat = '%.3f';
            app.uEditField_2.Editable = 'off';
            app.uEditField_2.HorizontalAlignment = 'center';
            app.uEditField_2.Position = [323 64 68 21];

            % Create vEditField_2Label
            app.vEditField_2Label = uilabel(app.ColorimetryofthelightPanel);
            app.vEditField_2Label.HorizontalAlignment = 'center';
            app.vEditField_2Label.Position = [298 33 25 22];
            app.vEditField_2Label.Text = 'v*';

            % Create vEditField_2
            app.vEditField_2 = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.vEditField_2.ValueDisplayFormat = '%.3f';
            app.vEditField_2.Editable = 'off';
            app.vEditField_2.HorizontalAlignment = 'center';
            app.vEditField_2.Position = [323 33 68 21];

            % Create aEditFieldLabel
            app.aEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.aEditFieldLabel.HorizontalAlignment = 'center';
            app.aEditFieldLabel.Position = [439 63 25 22];
            app.aEditFieldLabel.Text = 'a*';

            % Create aEditField
            app.aEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.aEditField.ValueDisplayFormat = '%.3f';
            app.aEditField.Editable = 'off';
            app.aEditField.HorizontalAlignment = 'center';
            app.aEditField.Position = [464 63 68 21];

            % Create bEditFieldLabel
            app.bEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.bEditFieldLabel.HorizontalAlignment = 'center';
            app.bEditFieldLabel.Position = [439 35 25 22];
            app.bEditFieldLabel.Text = 'b*';

            % Create bEditField
            app.bEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.bEditField.ValueDisplayFormat = '%.3f';
            app.bEditField.Editable = 'off';
            app.bEditField.HorizontalAlignment = 'center';
            app.bEditField.Position = [464 35 68 21];

            % Create CIE1976LabLabel
            app.CIE1976LabLabel = uilabel(app.ColorimetryofthelightPanel);
            app.CIE1976LabLabel.Position = [435 95 103 22];
            app.CIE1976LabLabel.Text = 'CIE 1976 (L* a* b*)';

            % Create LightnessLEditFieldLabel
            app.LightnessLEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.LightnessLEditFieldLabel.HorizontalAlignment = 'center';
            app.LightnessLEditFieldLabel.Position = [604 221 77 22];
            app.LightnessLEditFieldLabel.Text = 'Lightness (L*)';

            % Create LightnessLEditField
            app.LightnessLEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.LightnessLEditField.ValueDisplayFormat = '%.3f';
            app.LightnessLEditField.Editable = 'off';
            app.LightnessLEditField.HorizontalAlignment = 'center';
            app.LightnessLEditField.Position = [691 221 59 21];

            % Create ChromaCuvEditFieldLabel
            app.ChromaCuvEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.ChromaCuvEditFieldLabel.HorizontalAlignment = 'center';
            app.ChromaCuvEditFieldLabel.Position = [600 188 86 22];
            app.ChromaCuvEditFieldLabel.Text = 'Chroma  (C*uv)';

            % Create ChromaCuvEditField
            app.ChromaCuvEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.ChromaCuvEditField.ValueDisplayFormat = '%.3f';
            app.ChromaCuvEditField.Editable = 'off';
            app.ChromaCuvEditField.HorizontalAlignment = 'center';
            app.ChromaCuvEditField.Position = [691 188 59 21];

            % Create SaturationSuvEditFieldLabel
            app.SaturationSuvEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.SaturationSuvEditFieldLabel.HorizontalAlignment = 'center';
            app.SaturationSuvEditFieldLabel.Position = [596 153 94 22];
            app.SaturationSuvEditFieldLabel.Text = 'Saturation (S*uv)';

            % Create SaturationSuvEditField
            app.SaturationSuvEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.SaturationSuvEditField.ValueDisplayFormat = '%.3f';
            app.SaturationSuvEditField.Editable = 'off';
            app.SaturationSuvEditField.HorizontalAlignment = 'center';
            app.SaturationSuvEditField.Position = [691 153 59 21];

            % Create HueangleHuvEditFieldLabel
            app.HueangleHuvEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.HueangleHuvEditFieldLabel.HorizontalAlignment = 'center';
            app.HueangleHuvEditFieldLabel.Position = [596 116 94 22];
            app.HueangleHuvEditFieldLabel.Text = 'Hue angle (H*uv)';

            % Create HueangleHuvEditField
            app.HueangleHuvEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.HueangleHuvEditField.ValueDisplayFormat = '%.3f';
            app.HueangleHuvEditField.Editable = 'off';
            app.HueangleHuvEditField.HorizontalAlignment = 'center';
            app.HueangleHuvEditField.Position = [691 116 59 21];

            % Create NormalizedkCheckBox
            app.NormalizedkCheckBox = uicheckbox(app.ColorimetryofthelightPanel);
            app.NormalizedkCheckBox.ValueChangedFcn = createCallbackFcn(app, @NormalizedkCheckBoxValueChanged, true);
            app.NormalizedkCheckBox.Text = ' Normalized k';
            app.NormalizedkCheckBox.Position = [120 156 96 22];

            % Create ChromaticresponseLHurvichLabel
            app.ChromaticresponseLHurvichLabel = uilabel(app.ColorimetryofthelightPanel);
            app.ChromaticresponseLHurvichLabel.FontSize = 11;
            app.ChromaticresponseLHurvichLabel.Position = [335 228 163 22];
            app.ChromaticresponseLHurvichLabel.Text = 'Chromatic response (L. Hurvich)';

            % Create RedEditFieldLabel
            app.RedEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.RedEditFieldLabel.BackgroundColor = [1 0 0];
            app.RedEditFieldLabel.HorizontalAlignment = 'center';
            app.RedEditFieldLabel.Position = [357 200 39 22];
            app.RedEditFieldLabel.Text = 'Red';

            % Create RedEditField
            app.RedEditField = uieditfield(app.ColorimetryofthelightPanel, 'text');
            app.RedEditField.CharacterLimits = [0 10];
            app.RedEditField.Editable = 'off';
            app.RedEditField.HorizontalAlignment = 'center';
            app.RedEditField.Position = [406 200 60 22];

            % Create YellowEditFieldLabel
            app.YellowEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.YellowEditFieldLabel.BackgroundColor = [1 1 0];
            app.YellowEditFieldLabel.HorizontalAlignment = 'center';
            app.YellowEditFieldLabel.Position = [357 170 39 22];
            app.YellowEditFieldLabel.Text = 'Yellow';

            % Create YellowEditField
            app.YellowEditField = uieditfield(app.ColorimetryofthelightPanel, 'text');
            app.YellowEditField.CharacterLimits = [0 10];
            app.YellowEditField.Editable = 'off';
            app.YellowEditField.HorizontalAlignment = 'center';
            app.YellowEditField.Position = [406 170 60 22];

            % Create SaturationEditFieldLabel
            app.SaturationEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.SaturationEditFieldLabel.BackgroundColor = [1 1 1];
            app.SaturationEditFieldLabel.HorizontalAlignment = 'center';
            app.SaturationEditFieldLabel.Position = [336 135 60 22];
            app.SaturationEditFieldLabel.Text = 'Saturation';

            % Create SaturationEditField
            app.SaturationEditField = uieditfield(app.ColorimetryofthelightPanel, 'text');
            app.SaturationEditField.CharacterLimits = [0 10];
            app.SaturationEditField.Editable = 'off';
            app.SaturationEditField.HorizontalAlignment = 'center';
            app.SaturationEditField.Position = [406 135 60 22];

            % Create RrgEditFieldLabel
            app.RrgEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.RrgEditFieldLabel.HorizontalAlignment = 'center';
            app.RrgEditFieldLabel.Position = [480 200 31 22];
            app.RrgEditFieldLabel.Text = 'R r/g';

            % Create RrgEditField
            app.RrgEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.RrgEditField.ValueDisplayFormat = '%.3f';
            app.RrgEditField.Editable = 'off';
            app.RrgEditField.HorizontalAlignment = 'center';
            app.RrgEditField.Position = [518 200 58 23];

            % Create RybEditFieldLabel
            app.RybEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.RybEditFieldLabel.HorizontalAlignment = 'center';
            app.RybEditFieldLabel.Position = [479 167 34 22];
            app.RybEditFieldLabel.Text = 'R y/b';

            % Create RybEditField
            app.RybEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.RybEditField.ValueDisplayFormat = '%.3f';
            app.RybEditField.Editable = 'off';
            app.RybEditField.HorizontalAlignment = 'center';
            app.RybEditField.Position = [518 167 58 23];

            % Create Simulation
            app.Simulation = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.Simulation.ValueDisplayFormat = '%.0f';
            app.Simulation.HorizontalAlignment = 'center';
            app.Simulation.Position = [575 6 147 61];

            % Create REditFieldLabel
            app.REditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.REditFieldLabel.HorizontalAlignment = 'center';
            app.REditFieldLabel.Position = [731 56 25 18];
            app.REditFieldLabel.Text = 'R';

            % Create REditField
            app.REditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.REditField.ValueDisplayFormat = '%.0f';
            app.REditField.Editable = 'off';
            app.REditField.HorizontalAlignment = 'center';
            app.REditField.Position = [755 56 38 19];

            % Create GEditFieldLabel
            app.GEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.GEditFieldLabel.HorizontalAlignment = 'center';
            app.GEditFieldLabel.Position = [731 31 25 22];
            app.GEditFieldLabel.Text = 'G';

            % Create GEditField
            app.GEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.GEditField.ValueDisplayFormat = '%.0f';
            app.GEditField.Editable = 'off';
            app.GEditField.HorizontalAlignment = 'center';
            app.GEditField.Position = [755 33 38 19];

            % Create BEditFieldLabel
            app.BEditFieldLabel = uilabel(app.ColorimetryofthelightPanel);
            app.BEditFieldLabel.HorizontalAlignment = 'center';
            app.BEditFieldLabel.Position = [731 8 25 22];
            app.BEditFieldLabel.Text = 'B';

            % Create BEditField
            app.BEditField = uieditfield(app.ColorimetryofthelightPanel, 'numeric');
            app.BEditField.ValueDisplayFormat = '%.0f';
            app.BEditField.Editable = 'off';
            app.BEditField.HorizontalAlignment = 'center';
            app.BEditField.Position = [755 10 38 19];

            % Create SimulationLabel
            app.SimulationLabel = uilabel(app.ColorimetryofthelightPanel);
            app.SimulationLabel.FontSize = 11;
            app.SimulationLabel.Position = [625 69 57 22];
            app.SimulationLabel.Text = 'Simulation';

            % Create SerranoPedrazaI2023Label
            app.SerranoPedrazaI2023Label = uilabel(app.UIFigure);
            app.SerranoPedrazaI2023Label.FontSize = 9;
            app.SerranoPedrazaI2023Label.Position = [9 8 110 22];
            app.SerranoPedrazaI2023Label.Text = 'Serrano-Pedraza, I. (2023)';

            % Show the figure after all components are created
            app.UIFigure.Visible = 'on';
        end
    end

    % App creation and deletion
    methods (Access = public)

        % Construct app
        function app = Colorimetry_light

            % Create UIFigure and components
            createComponents(app)

            % Register the app with App Designer
            registerApp(app, app.UIFigure)

            % Execute the startup function
            runStartupFcn(app, @startupFcn)

            if nargout == 0
                clear app
            end
        end

        % Code that executes before app deletion
        function delete(app)

            % Delete UIFigure when app is deleted
            delete(app.UIFigure)
        end
    end
end