classdef Experiment_DSF_3D < matlab.apps.AppBase

    % Properties that correspond to app components
    properties (Access = public)
        UIFigure                        matlab.ui.Figure
        Label_5                         matlab.ui.control.Label
        DistanceEditFieldLabel_2        matlab.ui.control.Label
        Label_4                         matlab.ui.control.Label
        RightPanel                      matlab.ui.container.Panel
        REPEATButton                    matlab.ui.control.Button
        STOPButton                      matlab.ui.control.Button
        Label_3                         matlab.ui.control.Label
        DataanalysisButton              matlab.ui.control.Button
        of6Label                        matlab.ui.control.Label
        RestartexperimentButton         matlab.ui.control.Button
        NextstimulusButton              matlab.ui.control.Button
        PresentingthestimulusLamp       matlab.ui.control.Lamp
        PresentingthestimulusLampLabel  matlab.ui.control.Label
        StimulusEditField               matlab.ui.control.NumericEditField
        StimulusEditFieldLabel          matlab.ui.control.Label
        UIAxes                          matlab.ui.control.UIAxes
        LeftPanel                       matlab.ui.container.Panel
        yearsLabel                      matlab.ui.control.Label
        DistanceEditField               matlab.ui.control.NumericEditField
        DistanceEditFieldLabel_3        matlab.ui.control.Label
        StimulusSizeEditField           matlab.ui.control.NumericEditField
        StimulusSizeEditFieldLabel      matlab.ui.control.Label
        AgeEditField                    matlab.ui.control.NumericEditField
        AgeEditFieldLabel               matlab.ui.control.Label
        cmLabel_2                       matlab.ui.control.Label
        cmLabel                         matlab.ui.control.Label
        SerranoPedrazaI2023Label        matlab.ui.control.Label
        DIsparitySensitivityFunctionLabel  matlab.ui.control.Label
        InitialsEditField               matlab.ui.control.EditField
        StartExperimentButton           matlab.ui.control.Button
        InitialsEditFieldLabel          matlab.ui.control.Label
    end


    properties (Access = private)
        Data;
        trials;
        estimation;
        analysisApp;
        DialogApp;    % Dialog box app  
        spatialfrequency=2;
        STOP;
        logdisparity;
    end

    properties (Access = public)
        filename; % Description
    end
    
    methods (Access = private)
        
    end


    % Callbacks that handle component events
    methods (Access = private)

        % Button pushed function: StartExperimentButton
        function StartExperimentButtonPushed(app, event)
    %%%%Dr. Ignacio Serrano-Pedraza 2023
%%%iserrano@ucm.es
%%%%%Disparity Sensitivity Function 1D (Serrano-Pedraza et al. 2010)

%%%Serrano-Pedraza, I. & Read, J. C. A. (2010). Multiple channels for horizontal, but only one for vertical corrugations? A new look at the stereo anisotropy. 
%%%Journal of Vision, 10(12):10, 1–11, http://www.journalofvision.org/content/10/12/10, doi:10.1167/10.12.10.


    if app.StimulusSizeEditField.Value==0 || app.DistanceEditField.Value==0
        uialert(uifigure('Position',[100 100 500 250]),'Please, enter the stimulus size and the distance','Warning','Icon','warning');
    else


            app.STOP=0;
            app.StartExperimentButton.Enable = 'off' ;
            app.InitialsEditField.Enable = 'off' ;
            app.AgeEditField.Enable = 'off' ;
            app.StimulusSizeEditField.Enable='off';
            app.DistanceEditField.Enable='off';

            expt.age= app.AgeEditField.Value;
            expt.initials= app.InitialsEditField.Value;

            Trial= app.StimulusEditField.Value;
            if Trial ==1
                pause(1)
                mfile_name          = mfilename('fullpath');
                [pathstr,name,ext]  = fileparts(mfile_name);
                cd(pathstr);
                %%%    filename = strrep(strrep(['Data_' expt.initials '_' datestr(now) '.mat'],' ','_'),':','-');
                filename = strrep(strrep(['DATA_DSF_3D/Data_' expt.initials '_' datestr(now) '.xlsx'],' ','_'),':','-');
                app.filename=filename;
            end



            app.PresentingthestimulusLamp.Color = [0 1 0] ;
            app.NextstimulusButton.Enable = 'off' ;
            app.RestartexperimentButton.Enable = 'off' ;
            app.STOPButton.Enable='on';
            app.REPEATButton.Enable='off';

for i=1:255
    r(i)=i/255;
    g(i)=i/255;
    b(i)=i/255;
end
  gray2=[r',g',b'];


contador=0;
logdisparity=[2.4 2.2 1.8 1.4:-0.1:0.1 0.05 0.025 0.0125];
while contador<length(logdisparity)
contador=contador+1;
app.logdisparity=logdisparity(contador); 
TAMX=600;
TAMY=600;

expt.degreesfullscreenX=atand(app.StimulusSizeEditField.Value/2/app.DistanceEditField.Value);
ARCSEC=(10^app.logdisparity);
Disp2=(ARCSEC*TAMX)/(expt.degreesfullscreenX*3600);%%PIXELS  

frequencies=[1 2 3 5 8 12];%%CYCLES PER IMAGE
app.spatialfrequency=frequencies(Trial);

expt.numberofdots=4000;
expt.Orientation=90;
expt.visualangle=1;
phase=0;
sx=0.3;
sy=0.3;

dotXpix = TAMX*(rand(1, expt.numberofdots)-0.5);
dotYpix = TAMY*(rand(1, expt.numberofdots)-0.5);

Angulo=expt.Orientation*pi/180; 
xx=dotXpix.*cos(Angulo)+dotYpix.*sin(Angulo);
yy=-dotXpix.*sin(Angulo)+dotYpix.*cos(Angulo);

delta=Disp2*(cos(2*pi*xx*app.spatialfrequency*(expt.visualangle/TAMX)+phase)).*(exp(-((xx*(expt.visualangle/TAMX)).^2/(2*(sx).^2))-((yy*(expt.visualangle/TAMY)).^2/(2*(sy).^2))));  

dotXpix = TAMX/2+dotXpix;
dotYpix = TAMY/2+dotYpix;

jkeep = find((abs(dotXpix) > 10 & abs(dotXpix) < TAMX-10) & (abs(dotYpix) > 10 & abs(dotYpix) < TAMY-10));
dotXpix = dotXpix(jkeep);%
dotYpix = dotYpix(jkeep);%
delta=delta(jkeep);

dotxL = dotXpix+delta/2;
dotxR = dotXpix-delta/2;
dotyL= dotYpix;
dotyR= dotyL;


imageL=0.*ones(TAMX,TAMY);
imageR=0.*ones(TAMX,TAMY);

sx=2.5;
sy=sx;
tam=4;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%ANTIALISNG GAUSSIAN DOTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%(ISP)%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

NDOTS=0;
for i=1:length(dotxL)
m=1;
LL=imageL([round(dotyL(i))-tam:round(dotyL(i))+tam],[round(dotxL(i))-tam:round(dotxL(i))+tam]);
RR=imageR([round(dotyR(i))-tam:round(dotyR(i))+tam],[round(dotxR(i))-tam:round(dotxR(i))+tam]);
[H]=find(LL>0);
[G]=find(RR>0);
if isempty(H)==0 || isempty(G)==0 %%this eliminates overlaping of dots
else
[x,y]=meshgrid(round(dotxL(i))-tam:round(dotxL(i))+tam,round(dotyL(i))-tam:round(dotyL(i))+tam);
imageL([round(dotyL(i))-tam:round(dotyL(i))+tam],[round(dotxL(i))-tam:round(dotxL(i))+tam])=(m*(exp(-((x-dotxL(i)).^2/(2*(sx).^2))-((y-dotyL(i)).^2/(2*(sy).^2)))));
[x,y]=meshgrid(round(dotxR(i))-tam:round(dotxR(i))+tam,round(dotyR(i))-tam:round(dotyR(i))+tam);
imageR([round(dotyR(i))-tam:round(dotyR(i))+tam],[round(dotxR(i))-tam:round(dotxR(i))+tam])=(m*(exp(-((x-dotxR(i)).^2/(2*(sx).^2))-((y-dotyR(i)).^2/(2*(sy).^2)))));
NDOTS=NDOTS+1;
end
end


imagestereo(:,:,1)=imageR;
imagestereo(:,:,3)=0*ones(TAMX,TAMY);
imagestereo(:,:,2)=imageL;



duracion_segundos=0.5;
app.PresentingthestimulusLamp.Color = [1 0 0];
tiempo_inicio = tic;
while toc(tiempo_inicio) < duracion_segundos
noise=128*(1+2*(rand(TAMX,TAMY)-0.5));
image(app.UIAxes,noise)
colormap(app.UIAxes,gray2) 
axis(app.UIAxes,'off')
pause(0.1)
end

image(app.UIAxes,imagestereo)
%colormap(app.UIAxes,gray2) 
axis(app.UIAxes,'off')
app.PresentingthestimulusLamp.Color = [0 1 0] ;
pause(1)
app.PresentingthestimulusLamp.Color = [1 0 0] ;
tiempo_inicio = tic;
while toc(tiempo_inicio) < duracion_segundos
noise=128*(1+2*(rand(TAMX,TAMY)-0.5));
image(app.UIAxes,noise)
colormap(app.UIAxes,gray2) 
axis(app.UIAxes,'off')
pause(0.1)
end
            
            
if app.STOP==1
contador=length(logdisparity);
app.NextstimulusButton.Enable = 'on' ;
app.RestartexperimentButton.Enable = 'on' ;
app.REPEATButton.Enable='on';
end
if contador==length(logdisparity)
    app.REPEATButton.Enable='on';
end

end
    end
        end

        % Button pushed function: NextstimulusButton
        function NextstimulusButtonPushed(app, event)

      


                app.trials(app.StimulusEditField.Value)=app.StimulusEditField.Value;
                app.estimation(app.StimulusEditField.Value)=app.logdisparity;%%%

                data.trials=app.trials;
                data.estimation=app.estimation;
                Trials=[data.trials]';
                Thresholds=[data.estimation]';
                freq=[1 2 3 5 8 12];
                Frequencies_c_i=freq(1:app.StimulusEditField.Value)';
                Age= (app.AgeEditField.Value);
                Initials= string(app.InitialsEditField.Value);
                Stimulus_size=(app.StimulusSizeEditField.Value);
                Distance= (app.DistanceEditField.Value);
                %%%save(app.filename,'expt','data')
                T=table(Frequencies_c_i,Thresholds);
                writetable(T,app.filename,'Sheet',1,'Range','A1');
                T=table(Initials);
                writetable(T,app.filename,'Sheet',1,'Range','D1');
                T=table(Age,Stimulus_size,Distance);
                writetable(T,app.filename,'Sheet',1,'Range','E1');

                app.StimulusEditField.Value=app.StimulusEditField.Value+1;
                if app.StimulusEditField.Value==7%%%%Trials = 6. You have to put here 6+1
                    app.NextstimulusButton.Enable = 'off' ;
                    app.STOPButton.Enable='off';
                    app.REPEATButton.Enable='off';
                    app.DataanalysisButton.Enable = 'on' ;
                    app.DataanalysisButton.Visible = 'on' ;
                    app.StimulusEditField.Value=app.StimulusEditField.Value-1;

                else
                    app.STOP=0;
                    app.StartExperimentButtonPushed;

                end
      
          

        end

        % Button pushed function: RestartexperimentButton
        function RestartexperimentButtonPushed(app, event)
            app.StartExperimentButton.Enable = 'on' ;
            TAM=600;
            image_grid=254*ones(TAM,TAM);
            image_grid([1],1:TAM)=0;
            image_grid([TAM],1:TAM)=0;
            image_grid(1:TAM,[1])=0;
            image_grid(1:TAM,[TAM])=0;
            image(app.UIAxes,image_grid)
            colormap(app.UIAxes,gray) 
            hold(app.UIAxes,'on');
            gridx=linspace(0,TAM,10);
            for i=1:10
                plot(app.UIAxes,[gridx(i) gridx(i)],[min(gridx) max(gridx)],'k-','linewidth',0.5,'Color',[0.7 0.7 0.7])
                hold(app.UIAxes,'on');
                plot(app.UIAxes,[min(gridx) max(gridx)],[gridx(i) gridx(i)],'k-','linewidth',0.5,'Color',[0.7 0.7 0.7])
            end

            axis(app.UIAxes,'on')
            %set(get(app.UIAxes,'children'),'visible','off');
            app.InitialsEditField.Enable = 'on' ;
            app.AgeEditField.Enable = 'on' ;
            app.StimulusSizeEditField.Enable='on';
            app.DistanceEditField.Enable='on';
            app.NextstimulusButton.Enable = 'off' ;
            app.DataanalysisButton.Enable = 'off' ;
            app.DataanalysisButton.Visible = 'off' ;
            app.RestartexperimentButton.Enable = 'off';
            app.STOPButton.Enable='off';
            app.REPEATButton.Enable='off';
            app.StimulusEditField.Value=1;
            app.trials=[];
            app.estimation=[];
        end

        % Button pushed function: DataanalysisButton
        function DataanalysisButtonPushed(app, event)
            % Get sample size and colormap
            namefile = app.filename;
            app.DialogApp = DSF_Part_2(app,namefile);
        end

        % Button pushed function: STOPButton
        function STOPButtonPushed(app, event)
            app.STOP=1;
        end

        % Button pushed function: REPEATButton
        function REPEATButtonPushed(app, event)
                    app.STOP=0;
                    app.StartExperimentButtonPushed;


        end
    end

    % Component initialization
    methods (Access = private)

        % Create UIFigure and components
        function createComponents(app)

            % Create UIFigure and hide until all components are created
            app.UIFigure = uifigure('Visible', 'off');
            app.UIFigure.Position = [100 100 1035 834];
            app.UIFigure.Name = 'MATLAB App';

            % Create LeftPanel
            app.LeftPanel = uipanel(app.UIFigure);
            app.LeftPanel.Position = [9 447 254 367];

            % Create InitialsEditFieldLabel
            app.InitialsEditFieldLabel = uilabel(app.LeftPanel);
            app.InitialsEditFieldLabel.HorizontalAlignment = 'right';
            app.InitialsEditFieldLabel.Position = [53 274 39 22];
            app.InitialsEditFieldLabel.Text = 'Initials';

            % Create StartExperimentButton
            app.StartExperimentButton = uibutton(app.LeftPanel, 'push');
            app.StartExperimentButton.ButtonPushedFcn = createCallbackFcn(app, @StartExperimentButtonPushed, true);
            app.StartExperimentButton.FontSize = 18;
            app.StartExperimentButton.Position = [52 49 150 31];
            app.StartExperimentButton.Text = 'Start Experiment';

            % Create InitialsEditField
            app.InitialsEditField = uieditfield(app.LeftPanel, 'text');
            app.InitialsEditField.Position = [107 274 75 22];

            % Create DIsparitySensitivityFunctionLabel
            app.DIsparitySensitivityFunctionLabel = uilabel(app.LeftPanel);
            app.DIsparitySensitivityFunctionLabel.FontSize = 14;
            app.DIsparitySensitivityFunctionLabel.FontWeight = 'bold';
            app.DIsparitySensitivityFunctionLabel.Position = [29 329 200 22];
            app.DIsparitySensitivityFunctionLabel.Text = 'DIsparity Sensitivity Function';

            % Create SerranoPedrazaI2023Label
            app.SerranoPedrazaI2023Label = uilabel(app.LeftPanel);
            app.SerranoPedrazaI2023Label.FontSize = 9;
            app.SerranoPedrazaI2023Label.Position = [7 9 110 22];
            app.SerranoPedrazaI2023Label.Text = 'Serrano-Pedraza, I. (2023)';

            % Create cmLabel
            app.cmLabel = uilabel(app.LeftPanel);
            app.cmLabel.Position = [197 181 25 22];
            app.cmLabel.Text = 'cm';

            % Create cmLabel_2
            app.cmLabel_2 = uilabel(app.LeftPanel);
            app.cmLabel_2.Position = [197 131 25 22];
            app.cmLabel_2.Text = 'cm';

            % Create AgeEditFieldLabel
            app.AgeEditFieldLabel = uilabel(app.LeftPanel);
            app.AgeEditFieldLabel.HorizontalAlignment = 'right';
            app.AgeEditFieldLabel.Position = [66 231 26 22];
            app.AgeEditFieldLabel.Text = 'Age';

            % Create AgeEditField
            app.AgeEditField = uieditfield(app.LeftPanel, 'numeric');
            app.AgeEditField.Limits = [0 120];
            app.AgeEditField.ValueDisplayFormat = '%.0f';
            app.AgeEditField.Position = [107 231 76 22];

            % Create StimulusSizeEditFieldLabel
            app.StimulusSizeEditFieldLabel = uilabel(app.LeftPanel);
            app.StimulusSizeEditFieldLabel.HorizontalAlignment = 'right';
            app.StimulusSizeEditFieldLabel.Position = [24 181 77 22];
            app.StimulusSizeEditFieldLabel.Text = 'Stimulus Size';

            % Create StimulusSizeEditField
            app.StimulusSizeEditField = uieditfield(app.LeftPanel, 'numeric');
            app.StimulusSizeEditField.Limits = [0 Inf];
            app.StimulusSizeEditField.Position = [107 181 76 22];

            % Create DistanceEditFieldLabel_3
            app.DistanceEditFieldLabel_3 = uilabel(app.LeftPanel);
            app.DistanceEditFieldLabel_3.HorizontalAlignment = 'right';
            app.DistanceEditFieldLabel_3.Position = [43 131 52 22];
            app.DistanceEditFieldLabel_3.Text = 'Distance';

            % Create DistanceEditField
            app.DistanceEditField = uieditfield(app.LeftPanel, 'numeric');
            app.DistanceEditField.Limits = [0 Inf];
            app.DistanceEditField.Position = [107 131 76 22];

            % Create yearsLabel
            app.yearsLabel = uilabel(app.LeftPanel);
            app.yearsLabel.Position = [197 231 34 22];
            app.yearsLabel.Text = 'years';

            % Create RightPanel
            app.RightPanel = uipanel(app.UIFigure);
            app.RightPanel.FontAngle = 'italic';
            app.RightPanel.Position = [277 36 730 778];

            % Create UIAxes
            app.UIAxes = uiaxes(app.RightPanel);
            app.UIAxes.Toolbar.Visible = 'off';
            app.UIAxes.DataAspectRatio = [1 1 1];
            app.UIAxes.PlotBoxAspectRatio = [1 1 1];
            app.UIAxes.XTickLabel = '';
            app.UIAxes.YTickLabel = '';
            app.UIAxes.Box = 'on';
            app.UIAxes.XGrid = 'on';
            app.UIAxes.YGrid = 'on';
            app.UIAxes.Visible = 'off';
            app.UIAxes.Position = [88 188 555 519];

            % Create StimulusEditFieldLabel
            app.StimulusEditFieldLabel = uilabel(app.RightPanel);
            app.StimulusEditFieldLabel.BackgroundColor = [0.9412 0.9412 0.9412];
            app.StimulusEditFieldLabel.HorizontalAlignment = 'right';
            app.StimulusEditFieldLabel.FontSize = 14;
            app.StimulusEditFieldLabel.Position = [149 727 59 22];
            app.StimulusEditFieldLabel.Text = 'Stimulus';

            % Create StimulusEditField
            app.StimulusEditField = uieditfield(app.RightPanel, 'numeric');
            app.StimulusEditField.FontSize = 14;
            app.StimulusEditField.BackgroundColor = [0.9412 0.9412 0.9412];
            app.StimulusEditField.Position = [228 727 31 22];
            app.StimulusEditField.Value = 1;

            % Create PresentingthestimulusLampLabel
            app.PresentingthestimulusLampLabel = uilabel(app.RightPanel);
            app.PresentingthestimulusLampLabel.HorizontalAlignment = 'right';
            app.PresentingthestimulusLampLabel.FontSize = 14;
            app.PresentingthestimulusLampLabel.Position = [329 727 151 22];
            app.PresentingthestimulusLampLabel.Text = 'Presenting the stimulus';

            % Create PresentingthestimulusLamp
            app.PresentingthestimulusLamp = uilamp(app.RightPanel);
            app.PresentingthestimulusLamp.Position = [495 727 23 23];
            app.PresentingthestimulusLamp.Color = [1 0 0];

            % Create NextstimulusButton
            app.NextstimulusButton = uibutton(app.RightPanel, 'push');
            app.NextstimulusButton.ButtonPushedFcn = createCallbackFcn(app, @NextstimulusButtonPushed, true);
            app.NextstimulusButton.FontSize = 14;
            app.NextstimulusButton.Enable = 'off';
            app.NextstimulusButton.Position = [322 113 100 25];
            app.NextstimulusButton.Text = 'Next stimulus';

            % Create RestartexperimentButton
            app.RestartexperimentButton = uibutton(app.RightPanel, 'push');
            app.RestartexperimentButton.ButtonPushedFcn = createCallbackFcn(app, @RestartexperimentButtonPushed, true);
            app.RestartexperimentButton.FontSize = 14;
            app.RestartexperimentButton.Enable = 'off';
            app.RestartexperimentButton.Position = [553 26 134 25];
            app.RestartexperimentButton.Text = 'Restart experiment';

            % Create of6Label
            app.of6Label = uilabel(app.RightPanel);
            app.of6Label.Position = [267 727 25 22];
            app.of6Label.Text = 'of 6';

            % Create DataanalysisButton
            app.DataanalysisButton = uibutton(app.RightPanel, 'push');
            app.DataanalysisButton.ButtonPushedFcn = createCallbackFcn(app, @DataanalysisButtonPushed, true);
            app.DataanalysisButton.FontSize = 18;
            app.DataanalysisButton.FontWeight = 'bold';
            app.DataanalysisButton.FontColor = [0 0.4471 0.7412];
            app.DataanalysisButton.Enable = 'off';
            app.DataanalysisButton.Visible = 'off';
            app.DataanalysisButton.Position = [307 50 130 31];
            app.DataanalysisButton.Text = 'Data analysis';

            % Create Label_3
            app.Label_3 = uilabel(app.RightPanel);
            app.Label_3.WordWrap = 'on';
            app.Label_3.Enable = 'off';
            app.Label_3.Position = [68 58 159 80];
            app.Label_3.Text = 'Note: Press STOP when you don''t see the stimulus. Then press NEXT. If you want to see the stimulus again, press REPEAT.';

            % Create STOPButton
            app.STOPButton = uibutton(app.RightPanel, 'push');
            app.STOPButton.ButtonPushedFcn = createCallbackFcn(app, @STOPButtonPushed, true);
            app.STOPButton.FontSize = 14;
            app.STOPButton.Enable = 'off';
            app.STOPButton.Position = [259 146 100 25];
            app.STOPButton.Text = 'STOP';

            % Create REPEATButton
            app.REPEATButton = uibutton(app.RightPanel, 'push');
            app.REPEATButton.ButtonPushedFcn = createCallbackFcn(app, @REPEATButtonPushed, true);
            app.REPEATButton.FontSize = 14;
            app.REPEATButton.Enable = 'off';
            app.REPEATButton.Position = [395 146 100 25];
            app.REPEATButton.Text = 'REPEAT';

            % Create Label_4
            app.Label_4 = uilabel(app.UIFigure);
            app.Label_4.BackgroundColor = [1 1 1];
            app.Label_4.WordWrap = 'on';
            app.Label_4.Position = [33 201 207 177];
            app.Label_4.Text = 'You will see 6 sinusoidal corrugations (3D) of different spatial frequencies presented between 2D dynamic noises. In each presentation the stimulus will have lower disparity than the previous one. Your task will be to click the stop button when you don''t see the corrugations. After finish the experiment you will see a button "Data Analysis" that will take you to a different window where you will be able to analyze your data.';

            % Create DistanceEditFieldLabel_2
            app.DistanceEditFieldLabel_2 = uilabel(app.UIFigure);
            app.DistanceEditFieldLabel_2.HorizontalAlignment = 'right';
            app.DistanceEditFieldLabel_2.FontWeight = 'bold';
            app.DistanceEditFieldLabel_2.Position = [77 389 96 22];
            app.DistanceEditFieldLabel_2.Text = 'INSTRUCTIONS';

            % Create Label_5
            app.Label_5 = uilabel(app.UIFigure);
            app.Label_5.BackgroundColor = [1 1 1];
            app.Label_5.WordWrap = 'on';
            app.Label_5.FontColor = [1 0 0];
            app.Label_5.Position = [30 130 207 44];
            app.Label_5.Text = 'Important: You will need to wear anaglyph glasses and place the red filter in front of the right eye.';

            % Show the figure after all components are created
            app.UIFigure.Visible = 'on';
        end
    end

    % App creation and deletion
    methods (Access = public)

        % Construct app
        function app = Experiment_DSF_3D

            % Create UIFigure and components
            createComponents(app)

            % Register the app with App Designer
            registerApp(app, app.UIFigure)

            if nargout == 0
                clear app
            end
        end

        % Code that executes before app deletion
        function delete(app)

            % Delete UIFigure when app is deleted
            delete(app.UIFigure)
        end
    end
end