classdef Experiment_Stevens_Brightness < matlab.apps.AppBase

    % Properties that correspond to app components
    properties (Access = public)
        UIFigure                       matlab.ui.Figure
        ReferenceTestHighestintensityButton  matlab.ui.control.Button
        ReferenceTestLowestintensityButton  matlab.ui.control.Button
        Label_4                        matlab.ui.control.Label
        RightPanel                     matlab.ui.container.Panel
        BrightnessestimationEditFieldLabel_2  matlab.ui.control.Label
        Label_3                        matlab.ui.control.Label
        DataanalysisButton             matlab.ui.control.Button
        Label_2                        matlab.ui.control.Label
        of27Label                      matlab.ui.control.Label
        RestartButton                  matlab.ui.control.Button
        NexttrialButton                matlab.ui.control.Button
        RunningLamp                    matlab.ui.control.Lamp
        RunningLampLabel               matlab.ui.control.Label
        TrialEditField                 matlab.ui.control.NumericEditField
        TrialEditFieldLabel            matlab.ui.control.Label
        BrightnessestimationEditField  matlab.ui.control.NumericEditField
        UIAxes                         matlab.ui.control.UIAxes
        LeftPanel                      matlab.ui.container.Panel
        SerranoPedrazaI2023Label       matlab.ui.control.Label
        StevensBrightnessExperimentLabel  matlab.ui.control.Label
        AgeEditField                   matlab.ui.control.EditField
        AgeEditFieldLabel              matlab.ui.control.Label
        InitialsEditField              matlab.ui.control.EditField
        StartExperimentButton          matlab.ui.control.Button
        InitialsEditFieldLabel         matlab.ui.control.Label
    end


    properties (Access = private)
        Data;
        trials;
        estimation;
        analysisApp;
        DialogApp                   % Dialog box app
    end

    properties (Access = public)
        filename; % Description
    end


    % Callbacks that handle component events
    methods (Access = private)

        % Button pushed function: StartExperimentButton
        function StartExperimentButtonPushed(app, event)
            
            %%Dr. Ignacio Serrano-Pedraza 2023
            %%%The objective of this program is to estimate the parameters
            %%%of the Stevens' law for Brightness perception.
            % % Serrano-Pedraza, I., Sierra-Vázquez, V., & López-Báscuas, L.E. (2014). 
            % % Psicología de la Percepción. Prácticas. Ed. Síntesis. ISBN: 978-84-907704-6-7

            app.StartExperimentButton.Enable = 'off' ;
            app.InitialsEditField.Enable = 'off' ;
            app.AgeEditField.Enable = 'off' ;
            app.BrightnessestimationEditField.Enable = 'off' ;
            app.BrightnessestimationEditFieldLabel_2.Enable = 'off' ;
            app.ReferenceTestHighestintensityButton.Enable = 'off' ;
            app.ReferenceTestLowestintensityButton.Enable = 'off' ;

            expt.age= app.AgeEditField.Value;
            expt.initials= app.InitialsEditField.Value;

            Trial= app.TrialEditField.Value;
            if Trial ==1
                pause(1)
                mfile_name          = mfilename('fullpath');
                [pathstr,name,ext]  = fileparts(mfile_name);
                cd(pathstr);
                %%%    filename = strrep(strrep(['Data_' expt.initials '_' datestr(now) '.mat'],' ','_'),':','-');
                filename = strrep(strrep(['DATA_BRIGHTNESS/Data_' expt.initials '_' datestr(now) '.xlsx'],' ','_'),':','-');
                app.filename=filename;
            end



            app.RunningLamp.Color = [0 1 0] ;
            app.NexttrialButton.Enable = 'off' ;
            app.RestartButton.Enable = 'off' ;

gradosimagenx=8;
gradosimageny=8;
TAMX=400;
TAMY=400;
gx=gradosimagenx/TAMX;%Grados de angulo visual del pixel en la direccion x
gy=gradosimageny/TAMY;%Grados de angulo visual del pixel en la direccion y

[x,y]=meshgrid(-gradosimagenx/2:(gradosimagenx/TAMX):((gradosimagenx/2)-(gradosimagenx/TAMX)),((gradosimageny/2)):-(gradosimageny/TAMY):-((gradosimageny/2)-(gradosimageny/TAMY)));
Angulo=90*pi/180; 
xx=x.*cos(Angulo)+y.*sin(Angulo);
yy=-x.*sin(Angulo)+y.*cos(Angulo);

for i=1:255
    r(i)=i/255;
    g(i)=i/255;
    b(i)=i/255;
end
  gray2=[r',g',b'];


L=[0:1/255:1];
LUT=[L'];

a=-0.0060;
b=-0.1743;
k=1.1772;
g=2.3026;
invG=1/g;


Contrast=1;
LUT2=(0.5*(1+Contrast*((128-0.5)./0.5)))/256;
LUT3=255*((real((LUT2-a).^invG))-b)./k;%%%Pelli & Zhang (1991) Eqs. 9 & 10.
ventana1=0.*ones(400,400);
for i=1:400
   for j=1:400
      rho=sqrt(xx(i,j).^2+yy(i,j).^2);

      if rho<=2.5
         ventana1(i,j)=LUT3;
      end
   end
end

Test=[192   128   224    32   224   192   128    64    64   248   248   224   240 8    32   240   128   248    16    16     8   192     8    64    16    32   240];

LUT2=(0.5*(1+Contrast*((Test(Trial)-0.5)./0.5)))/256;
LUT3=255*((real((LUT2-a).^invG))-b)./k;
ventana2=0.*ones(400,400);
for i=1:400
   for j=1:400
      rho=sqrt(xx(i,j).^2+yy(i,j).^2);

      if rho<=2.5
         ventana2(i,j)=LUT3;
      end
   end
end
imagen=[ventana1 ventana2];

noise=128*(1+2*(rand(800,400)-0.5));

image(app.UIAxes,noise)
colormap(app.UIAxes,gray2) 
axis(app.UIAxes,'off')
%app.UIAxes.Visible = 'on' ;
pause(2)

image(app.UIAxes,imagen)
colormap(app.UIAxes,gray2) 
axis(app.UIAxes,'off')
%app.UIAxes.Visible = 'on' ;

noise=128*(1+2*(rand(800,400)-0.5));
pause(2)

image(app.UIAxes,noise)
colormap(app.UIAxes,gray2) 
axis(app.UIAxes,'off')
%app.UIAxes.Visible = 'on' ;
            app.RunningLamp.Color = [1 0 0] ;
            app.BrightnessestimationEditField.Value=0;
            app.NexttrialButton.Enable = 'on' ;
            app.Label_3.Enable = 'on' ;
            app.Label_2.Enable = 'on' ;
            app.RestartButton.Enable = 'on' ;
            app.BrightnessestimationEditField.Enable = 'on' ;
            app.BrightnessestimationEditFieldLabel_2.Enable = 'on' ;



        end

        % Button pushed function: NexttrialButton
        function NexttrialButtonPushed(app, event)

            if app.BrightnessestimationEditField.Value>0

                app.Label_2.Enable = 'off' ;
                app.Label_3.Enable = 'off' ;
                app.trials(app.TrialEditField.Value)=app.TrialEditField.Value;
                app.estimation(app.TrialEditField.Value)=app.BrightnessestimationEditField.Value;

                data.trials=app.trials;
                data.estimation=app.estimation;
                Trials=[data.trials]';
                Estimations=[data.estimation]';
                Test=[192   128   224    32   224   192   128    64    64   248   248   224   240 8    32   240   128   248    16    16     8   192     8    64    16    32   240];
                Luminance_test=Test(1:app.TrialEditField.Value)';
                Age= string(app.AgeEditField.Value);
                Initials= string(app.InitialsEditField.Value);
                %%%save(app.filename,'expt','data')
                T=table(Trials,Luminance_test,Estimations);
                writetable(T,app.filename,'Sheet',1,'Range','A1');
                T=table(Age,Initials);
                writetable(T,app.filename,'Sheet',1,'Range','D1');
                app.TrialEditField.Value=app.TrialEditField.Value+1;
                if app.TrialEditField.Value==28%%%%Trials = 27. You have to put here 27+1
                    app.NexttrialButton.Enable = 'off' ;
                    app.BrightnessestimationEditField.Enable = 'off' ;
                    app.BrightnessestimationEditFieldLabel_2.Enable = 'off' ;
                    app.Label_3.Enable = 'off' ;
                    app.Label_2.Enable = 'off' ;
                    app.DataanalysisButton.Enable = 'on' ;
                    app.DataanalysisButton.Visible = 'on' ;
                    app.TrialEditField.Value=app.TrialEditField.Value-1;

                else
                    app.StartExperimentButtonPushed;
                end
            else
            end

        end

        % Button pushed function: RestartButton
        function RestartButtonPushed(app, event)
            app.StartExperimentButton.Enable = 'on' ;
            %image(app.UIAxes,128*ones(800,400))
            %app.UIAxes.Visible = 'off' ;
            set(app.UIAxes,'visible','off'); 
            set(get(app.UIAxes,'children'),'visible','off');
            app.InitialsEditField.Enable = 'on' ;
            app.AgeEditField.Enable = 'on' ;
            app.ReferenceTestHighestintensityButton.Enable = 'on' ;
            app.ReferenceTestLowestintensityButton.Enable = 'on' ;
            app.BrightnessestimationEditField.Enable = 'off' ;
            app.BrightnessestimationEditFieldLabel_2.Enable = 'off' ;
            app.NexttrialButton.Enable = 'off' ;
            app.DataanalysisButton.Enable = 'off' ;
            app.DataanalysisButton.Visible = 'off' ;
            app.Label_2.Enable = 'off' ;
            app.Label_3.Enable = 'off' ;
            app.RestartButton.Enable = 'off';
            app.TrialEditField.Value=1;
            app.BrightnessestimationEditField.Value=0;
            app.AgeEditField.Value='';
            app.InitialsEditField.Value='';
            app.trials=[];
            app.estimation=[];
        end

        % Button pushed function: DataanalysisButton
        function DataanalysisButtonPushed(app, event)
            % Get sample size and colormap
            namefile = app.filename;
            app.DialogApp = Brightness_Part_2(app,namefile);
        end

        % Button pushed function: ReferenceTestLowestintensityButton
        function ReferenceTestLowestintensityButtonPushed(app, event)
                        gradosimagenx=8;
gradosimageny=8;
TAMX=400;
TAMY=400;
gx=gradosimagenx/TAMX;%Grados de angulo visual del pixel en la direccion x
gy=gradosimageny/TAMY;%Grados de angulo visual del pixel en la direccion y

[x,y]=meshgrid(-gradosimagenx/2:(gradosimagenx/TAMX):((gradosimagenx/2)-(gradosimagenx/TAMX)),((gradosimageny/2)):-(gradosimageny/TAMY):-((gradosimageny/2)-(gradosimageny/TAMY)));
Angulo=90*pi/180; 
xx=x.*cos(Angulo)+y.*sin(Angulo);
yy=-x.*sin(Angulo)+y.*cos(Angulo);

for i=1:255
    r(i)=i/255;
    g(i)=i/255;
    b(i)=i/255;
end
  gray2=[r',g',b'];


L=[0:1/255:1];
LUT=[L'];

a=-0.0060;
b=-0.1743;
k=1.1772;
g=2.3026;
invG=1/g;


Contrast=1;
LUT2=(0.5*(1+Contrast*((128-0.5)./0.5)))/256;
LUT3=255*((real((LUT2-a).^invG))-b)./k;
ventana1=0.*ones(400,400);
for i=1:400
   for j=1:400
      rho=sqrt(xx(i,j).^2+yy(i,j).^2);

      if rho<=2.5
         ventana1(i,j)=LUT3;
      end
   end
end

Test=[ 8 ];

LUT2=(0.5*(1+Contrast*((Test-0.5)./0.5)))/256;
LUT3=255*((real((LUT2-a).^invG))-b)./k;
ventana2=0.*ones(400,400);
for i=1:400
   for j=1:400
      rho=sqrt(xx(i,j).^2+yy(i,j).^2);

      if rho<=2.5
         ventana2(i,j)=LUT3;
      end
   end
end
imagen=[ventana1 ventana2];

image(app.UIAxes,imagen)
colormap(app.UIAxes,gray2) 
axis(app.UIAxes,'off')
        end

        % Button pushed function: ReferenceTestHighestintensityButton
        function ReferenceTestHighestintensityButtonPushed(app, event)
                        gradosimagenx=8;
gradosimageny=8;
TAMX=400;
TAMY=400;
gx=gradosimagenx/TAMX;%Grados de angulo visual del pixel en la direccion x
gy=gradosimageny/TAMY;%Grados de angulo visual del pixel en la direccion y

[x,y]=meshgrid(-gradosimagenx/2:(gradosimagenx/TAMX):((gradosimagenx/2)-(gradosimagenx/TAMX)),((gradosimageny/2)):-(gradosimageny/TAMY):-((gradosimageny/2)-(gradosimageny/TAMY)));
Angulo=90*pi/180; 
xx=x.*cos(Angulo)+y.*sin(Angulo);
yy=-x.*sin(Angulo)+y.*cos(Angulo);

for i=1:255
    r(i)=i/255;
    g(i)=i/255;
    b(i)=i/255;
end
  gray2=[r',g',b'];


L=[0:1/255:1];
LUT=[L'];

a=-0.0060;
b=-0.1743;
k=1.1772;
g=2.3026;
invG=1/g;


Contrast=1;
LUT2=(0.5*(1+Contrast*((128-0.5)./0.5)))/256;
LUT3=255*((real((LUT2-a).^invG))-b)./k;
ventana1=0.*ones(400,400);
for i=1:400
   for j=1:400
      rho=sqrt(xx(i,j).^2+yy(i,j).^2);

      if rho<=2.5
         ventana1(i,j)=LUT3;
      end
   end
end

Test=[ 248 ];

LUT2=(0.5*(1+Contrast*((Test-0.5)./0.5)))/256;
LUT3=255*((real((LUT2-a).^invG))-b)./k;
ventana2=0.*ones(400,400);
for i=1:400
   for j=1:400
      rho=sqrt(xx(i,j).^2+yy(i,j).^2);

      if rho<=2.5
         ventana2(i,j)=LUT3;
      end
   end
end
imagen=[ventana1 ventana2];

image(app.UIAxes,imagen)
colormap(app.UIAxes,gray2) 
axis(app.UIAxes,'off')
        end
    end

    % Component initialization
    methods (Access = private)

        % Create UIFigure and components
        function createComponents(app)

            % Create UIFigure and hide until all components are created
            app.UIFigure = uifigure('Visible', 'off');
            app.UIFigure.Position = [100 100 904 631];
            app.UIFigure.Name = 'MATLAB App';

            % Create LeftPanel
            app.LeftPanel = uipanel(app.UIFigure);
            app.LeftPanel.Position = [9 367 254 244];

            % Create InitialsEditFieldLabel
            app.InitialsEditFieldLabel = uilabel(app.LeftPanel);
            app.InitialsEditFieldLabel.HorizontalAlignment = 'right';
            app.InitialsEditFieldLabel.Position = [42 151 39 22];
            app.InitialsEditFieldLabel.Text = 'Initials';

            % Create StartExperimentButton
            app.StartExperimentButton = uibutton(app.LeftPanel, 'push');
            app.StartExperimentButton.ButtonPushedFcn = createCallbackFcn(app, @StartExperimentButtonPushed, true);
            app.StartExperimentButton.FontSize = 18;
            app.StartExperimentButton.Position = [52 55 150 31];
            app.StartExperimentButton.Text = 'Start Experiment';

            % Create InitialsEditField
            app.InitialsEditField = uieditfield(app.LeftPanel, 'text');
            app.InitialsEditField.Position = [96 151 75 22];

            % Create AgeEditFieldLabel
            app.AgeEditFieldLabel = uilabel(app.LeftPanel);
            app.AgeEditFieldLabel.HorizontalAlignment = 'right';
            app.AgeEditFieldLabel.Position = [52 108 26 22];
            app.AgeEditFieldLabel.Text = 'Age';

            % Create AgeEditField
            app.AgeEditField = uieditfield(app.LeftPanel, 'text');
            app.AgeEditField.CharacterLimits = [0 150];
            app.AgeEditField.InputType = 'digits';
            app.AgeEditField.Position = [97 108 74 22];

            % Create StevensBrightnessExperimentLabel
            app.StevensBrightnessExperimentLabel = uilabel(app.LeftPanel);
            app.StevensBrightnessExperimentLabel.FontSize = 14;
            app.StevensBrightnessExperimentLabel.FontWeight = 'bold';
            app.StevensBrightnessExperimentLabel.Position = [29 206 219 22];
            app.StevensBrightnessExperimentLabel.Text = 'Stevens'' Brightness Experiment';

            % Create SerranoPedrazaI2023Label
            app.SerranoPedrazaI2023Label = uilabel(app.LeftPanel);
            app.SerranoPedrazaI2023Label.FontSize = 9;
            app.SerranoPedrazaI2023Label.Position = [14 12 110 22];
            app.SerranoPedrazaI2023Label.Text = 'Serrano-Pedraza, I. (2023)';

            % Create RightPanel
            app.RightPanel = uipanel(app.UIFigure);
            app.RightPanel.FontAngle = 'italic';
            app.RightPanel.Position = [277 25 592 586];

            % Create UIAxes
            app.UIAxes = uiaxes(app.RightPanel);
            app.UIAxes.Toolbar.Visible = 'off';
            app.UIAxes.PlotBoxAspectRatio = [2 1 1];
            app.UIAxes.Visible = 'off';
            app.UIAxes.Position = [17 179 605 357];

            % Create BrightnessestimationEditField
            app.BrightnessestimationEditField = uieditfield(app.RightPanel, 'numeric');
            app.BrightnessestimationEditField.Limits = [0 300];
            app.BrightnessestimationEditField.RoundFractionalValues = 'on';
            app.BrightnessestimationEditField.ValueDisplayFormat = '%.0f';
            app.BrightnessestimationEditField.FontSize = 14;
            app.BrightnessestimationEditField.Enable = 'off';
            app.BrightnessestimationEditField.Position = [310 166 53 22];

            % Create TrialEditFieldLabel
            app.TrialEditFieldLabel = uilabel(app.RightPanel);
            app.TrialEditFieldLabel.BackgroundColor = [0.9412 0.9412 0.9412];
            app.TrialEditFieldLabel.HorizontalAlignment = 'right';
            app.TrialEditFieldLabel.FontSize = 14;
            app.TrialEditFieldLabel.Position = [178 535 30 22];
            app.TrialEditFieldLabel.Text = 'Trial';

            % Create TrialEditField
            app.TrialEditField = uieditfield(app.RightPanel, 'numeric');
            app.TrialEditField.FontSize = 14;
            app.TrialEditField.BackgroundColor = [0.9412 0.9412 0.9412];
            app.TrialEditField.Position = [228 535 31 22];
            app.TrialEditField.Value = 1;

            % Create RunningLampLabel
            app.RunningLampLabel = uilabel(app.RightPanel);
            app.RunningLampLabel.HorizontalAlignment = 'right';
            app.RunningLampLabel.FontSize = 14;
            app.RunningLampLabel.Position = [310 535 57 22];
            app.RunningLampLabel.Text = 'Running';

            % Create RunningLamp
            app.RunningLamp = uilamp(app.RightPanel);
            app.RunningLamp.Position = [382 535 23 23];
            app.RunningLamp.Color = [1 0 0];

            % Create NexttrialButton
            app.NexttrialButton = uibutton(app.RightPanel, 'push');
            app.NexttrialButton.ButtonPushedFcn = createCallbackFcn(app, @NexttrialButtonPushed, true);
            app.NexttrialButton.FontSize = 14;
            app.NexttrialButton.Enable = 'off';
            app.NexttrialButton.Position = [257 122 100 25];
            app.NexttrialButton.Text = 'Next trial';

            % Create RestartButton
            app.RestartButton = uibutton(app.RightPanel, 'push');
            app.RestartButton.ButtonPushedFcn = createCallbackFcn(app, @RestartButtonPushed, true);
            app.RestartButton.FontSize = 14;
            app.RestartButton.Enable = 'off';
            app.RestartButton.Position = [440 35 64 25];
            app.RestartButton.Text = 'Restart';

            % Create of27Label
            app.of27Label = uilabel(app.RightPanel);
            app.of27Label.Position = [267 535 32 22];
            app.of27Label.Text = 'of 27';

            % Create Label_2
            app.Label_2 = uilabel(app.RightPanel);
            app.Label_2.FontSize = 14;
            app.Label_2.Enable = 'off';
            app.Label_2.Position = [373 165 49 22];
            app.Label_2.Text = '(1-300)';

            % Create DataanalysisButton
            app.DataanalysisButton = uibutton(app.RightPanel, 'push');
            app.DataanalysisButton.ButtonPushedFcn = createCallbackFcn(app, @DataanalysisButtonPushed, true);
            app.DataanalysisButton.FontSize = 18;
            app.DataanalysisButton.FontWeight = 'bold';
            app.DataanalysisButton.FontColor = [0 0.4471 0.7412];
            app.DataanalysisButton.Enable = 'off';
            app.DataanalysisButton.Visible = 'off';
            app.DataanalysisButton.Position = [242 59 130 31];
            app.DataanalysisButton.Text = 'Data analysis';

            % Create Label_3
            app.Label_3 = uilabel(app.RightPanel);
            app.Label_3.WordWrap = 'on';
            app.Label_3.Enable = 'off';
            app.Label_3.Position = [35 67 159 80];
            app.Label_3.Text = 'Note: Reference stimulus is always presented on the left. The Brightness estimation for the Reference stimulus is 100.';

            % Create BrightnessestimationEditFieldLabel_2
            app.BrightnessestimationEditFieldLabel_2 = uilabel(app.RightPanel);
            app.BrightnessestimationEditFieldLabel_2.HorizontalAlignment = 'right';
            app.BrightnessestimationEditFieldLabel_2.FontSize = 14;
            app.BrightnessestimationEditFieldLabel_2.Enable = 'off';
            app.BrightnessestimationEditFieldLabel_2.Position = [158 165 141 22];
            app.BrightnessestimationEditFieldLabel_2.Text = 'Brightness estimation';

            % Create Label_4
            app.Label_4 = uilabel(app.UIFigure);
            app.Label_4.WordWrap = 'on';
            app.Label_4.Position = [38 261 207 80];
            app.Label_4.Text = 'Check the lowest and the highest Test stimulus compared with the Reference estimulus. ';

            % Create ReferenceTestLowestintensityButton
            app.ReferenceTestLowestintensityButton = uibutton(app.UIFigure, 'push');
            app.ReferenceTestLowestintensityButton.ButtonPushedFcn = createCallbackFcn(app, @ReferenceTestLowestintensityButtonPushed, true);
            app.ReferenceTestLowestintensityButton.Position = [38 227 200 23];
            app.ReferenceTestLowestintensityButton.Text = 'Reference - Test (Lowest intensity)';

            % Create ReferenceTestHighestintensityButton
            app.ReferenceTestHighestintensityButton = uibutton(app.UIFigure, 'push');
            app.ReferenceTestHighestintensityButton.ButtonPushedFcn = createCallbackFcn(app, @ReferenceTestHighestintensityButtonPushed, true);
            app.ReferenceTestHighestintensityButton.Position = [38 166 202 23];
            app.ReferenceTestHighestintensityButton.Text = 'Reference - Test (Highest intensity)';

            % Show the figure after all components are created
            app.UIFigure.Visible = 'on';
        end
    end

    % App creation and deletion
    methods (Access = public)

        % Construct app
        function app = Experiment_Stevens_Brightness

            % Create UIFigure and components
            createComponents(app)

            % Register the app with App Designer
            registerApp(app, app.UIFigure)

            if nargout == 0
                clear app
            end
        end

        % Code that executes before app deletion
        function delete(app)

            % Delete UIFigure when app is deleted
            delete(app.UIFigure)
        end
    end
end