classdef Experiment_Stevens_Loudness < matlab.apps.AppBase

    % Properties that correspond to app components
    properties (Access = public)
        UIFigure                        matlab.ui.Figure
        RightPanel                      matlab.ui.container.Panel
        Label_3                         matlab.ui.control.Label
        LoudnessestimationEditFieldLabel_2  matlab.ui.control.Label
        DataanalysisButton              matlab.ui.control.Button
        Label_2                         matlab.ui.control.Label
        of27Label                       matlab.ui.control.Label
        RestartButton                   matlab.ui.control.Button
        NexttrialButton                 matlab.ui.control.Button
        RunningLamp                     matlab.ui.control.Lamp
        RunningLampLabel                matlab.ui.control.Label
        TrialEditField                  matlab.ui.control.NumericEditField
        TrialEditFieldLabel             matlab.ui.control.Label
        LoudnessestimationEditField     matlab.ui.control.NumericEditField
        LeftPanel                       matlab.ui.container.Panel
        PlayreferencesoundButton        matlab.ui.control.Button
        SerranoPedrazaI2023Label        matlab.ui.control.Label
        PlayloudestsoundButton          matlab.ui.control.Button
        Label                           matlab.ui.control.Label
        StevensLoudnessExperimentLabel  matlab.ui.control.Label
        AdjustthevolumesothelowestnoiseisbarelyaudibleLabel  matlab.ui.control.Label
        SoundcalibrationLabel           matlab.ui.control.Label
        PlaylowestsoundButton           matlab.ui.control.Button
        Knob                            matlab.ui.control.Knob
        AgeEditField                    matlab.ui.control.EditField
        AgeEditFieldLabel               matlab.ui.control.Label
        InitialsEditField               matlab.ui.control.EditField
        StartExperimentButton           matlab.ui.control.Button
        InitialsEditFieldLabel          matlab.ui.control.Label
    end


    properties (Access = private)
        Data;
        trials;
        estimation;
        analysisApp;
        DialogApp                   % Dialog box app
    end

    properties (Access = public)
        filename; % Description
    end


    % Callbacks that handle component events
    methods (Access = private)

        % Button pushed function: StartExperimentButton
        function StartExperimentButtonPushed(app, event)


            %%%Dr. Ignacio Serrano-Pedraza 2023
            %%%Experiment to estimate the parameters of the Stevens' Law for Loudness.
            % % Serrano-Pedraza, I., Sierra-Vázquez, V., & López-Báscuas, L.E. (2014). 
            % % Psicología de la Percepción. Prácticas. Ed. Síntesis. ISBN: 978-84-907704-6-7

            app.StartExperimentButton.Enable = 'off' ;
            app.PlaylowestsoundButton.Enable = 'off' ;
            app.PlayloudestsoundButton.Enable = 'off' ;
            app.PlayreferencesoundButton.Enable = 'off' ;
            app.Knob.Enable = 'off' ;
            app.InitialsEditField.Enable = 'off' ;
            app.AgeEditField.Enable = 'off' ;
            app.LoudnessestimationEditField.Enable = 'off' ;
            app.LoudnessestimationEditFieldLabel_2.Enable = 'off' ;
            app.Label_2.Enable = 'off' ;
            app.Label_3.Enable = 'on' ;

            volumen1 = app.Knob.Value;
            expt.age= app.AgeEditField.Value;
            expt.initials= app.InitialsEditField.Value;

            Trial= app.TrialEditField.Value;
            if Trial ==1
                pause(1)
                mfile_name          = mfilename('fullpath');
                [pathstr,name,ext]  = fileparts(mfile_name);
                cd(pathstr);
                %%%    filename = strrep(strrep(['Data_' expt.initials '_' datestr(now) '.mat'],' ','_'),':','-');
                filename = strrep(strrep(['DATA_LOUDNESS/Data_' expt.initials '_' datestr(now) '.xlsx'],' ','_'),':','-');
                app.filename=filename;
            end

            DECIBELS=[-20 -15 -10 -5 0 5 10 15 20];
            positions=[5 3 9 5 2 8 6 8 9 4 1 6 7 2 4 6 1 2 7 3 9 4 8 5 7 1 3];%%
            samplingrate=44100;
            duration=1; %seconds
            t=[0+(1/samplingrate):1/samplingrate:duration]';
            n=length(t);
            freq=(-fix(n/2):fix((n-1)/2))/duration;

            %%%Reference
            L0_R=volumen1+20;
            N0=10^-12*(10^(L0_R/10));%%pp. 48 Hartmann
            CRMS=sqrt(N0*max(freq));%%pp. 48 Hartmann
            R= 0 + CRMS * randn(length(t),1);

            %%Silence
            S= 0 + 0 * randn(length(t),1);

            %%Test
            L0=L0_R+DECIBELS(positions(Trial));
            N0=10^-12*(10^(L0/10));%%pp. 48 Hartmann
            CRMS=sqrt(N0*max(freq));%%pp. 48 Hartmann
            T= 0 + CRMS * randn(length(t),1);


            app.RunningLamp.Color = [0 1 0] ;
            app.NexttrialButton.Enable = 'off' ;
            app.RestartButton.Enable = 'off' ;

            s=[S;R;S;T];
            sound(s,samplingrate)
            pause(4*duration)
            app.RunningLamp.Color = [1 0 0] ;
            app.LoudnessestimationEditField.Value=0;
            app.NexttrialButton.Enable = 'on' ;
            app.RestartButton.Enable = 'on' ;
            app.LoudnessestimationEditField.Enable = 'on' ;
            app.LoudnessestimationEditFieldLabel_2.Enable = 'on' ;
            app.Label_2.Enable = 'on' ;


        end

        % Button pushed function: PlaylowestsoundButton
        function PlaylowestsoundButtonPushed(app, event)
            volumen1 = app.Knob.Value;
            samplingrate=44100;
            duration=1; %seconds
            t=[0+(1/samplingrate):1/samplingrate:duration]';
            n=length(t);
            freq=(-fix(n/2):fix((n-1)/2))/duration;

            %%%Noise with lowest volume
            L0=volumen1;
            N0=10^-12*(10^(L0/10));%%pp. 48 Hartmann
            CRMS=sqrt(N0*max(freq));%%pp. 48 Hartmann
            R= 0 + CRMS * randn(length(t),1);
            sound(R,samplingrate)
        end

        % Button pushed function: NexttrialButton
        function NexttrialButtonPushed(app, event)

            if app.LoudnessestimationEditField.Value>0

                app.trials(app.TrialEditField.Value)=app.TrialEditField.Value;
                app.estimation(app.TrialEditField.Value)=app.LoudnessestimationEditField.Value;

                data.trials=app.trials;
                data.estimation=app.estimation;
                Trials=[data.trials]';
                Estimations=[data.estimation]';
                DECIBELS=[-20 -15 -10 -5 0 5 10 15 20];
                positions=[5 3 9 5 2 8 6 8 9 4 1 6 7 2 4 6 1 2 7 3 9 4 8 5 7 1 3];%%
                Difference_dB=DECIBELS(positions)';
                Difference_in_dB=Difference_dB(1:app.TrialEditField.Value);
                Age= string(app.AgeEditField.Value);
                Initials= string(app.InitialsEditField.Value);
                %%%save(app.filename,'expt','data')
                T=table(Trials,Estimations,Difference_in_dB);
                writetable(T,app.filename,'Sheet',1,'Range','A1');
                T=table(Age,Initials);
                writetable(T,app.filename,'Sheet',1,'Range','D1');
                app.TrialEditField.Value=app.TrialEditField.Value+1;
                if app.TrialEditField.Value==28%%%%Trials = 27. You have to put here 27+1
                    app.NexttrialButton.Enable = 'off' ;
                    app.LoudnessestimationEditField.Enable = 'off' ;
                    app.LoudnessestimationEditFieldLabel_2.Enable = 'off' ;
                    app.Label_2.Enable = 'off' ;
                    app.DataanalysisButton.Enable = 'on' ;
                    app.DataanalysisButton.Visible = 'on' ;
                    app.TrialEditField.Value=app.TrialEditField.Value-1;

                else
                    app.StartExperimentButtonPushed;
                end
            else
            end

        end

        % Button pushed function: RestartButton
        function RestartButtonPushed(app, event)
            app.StartExperimentButton.Enable = 'on' ;
            app.PlaylowestsoundButton.Enable = 'on' ;
            app.PlayloudestsoundButton.Enable = 'on' ;
            app.PlayreferencesoundButton.Enable = 'on' ;
            app.Knob.Enable = 'on' ;
            app.InitialsEditField.Enable = 'on' ;
            app.AgeEditField.Enable = 'on' ;
            app.LoudnessestimationEditField.Enable = 'off' ;
            app.LoudnessestimationEditFieldLabel_2.Enable = 'off' ;
            app.Label_2.Enable = 'off' ;
            app.Label_3.Enable = 'off' ;
            app.NexttrialButton.Enable = 'off' ;
            app.DataanalysisButton.Enable = 'off' ;
            app.DataanalysisButton.Visible = 'off' ;
            app.TrialEditField.Value=1;
            app.LoudnessestimationEditField.Value=0;
            app.AgeEditField.Value='';
            app.InitialsEditField.Value='';
            app.trials=[];
            app.estimation=[];
        end

        % Button pushed function: PlayloudestsoundButton
        function PlayloudestsoundButtonPushed(app, event)
            volumen1 = app.Knob.Value+40;
            samplingrate=44100;
            duration=1; %seconds
            t=[0+(1/samplingrate):1/samplingrate:duration]';
            n=length(t);
            freq=(-fix(n/2):fix((n-1)/2))/duration;

            %%%Noise with loudest volume
            L0=volumen1;
            N0=10^-12*(10^(L0/10));%%pp. 48 Hartmann
            CRMS=sqrt(N0*max(freq));%%pp. 48 Hartmann
            R= 0 + CRMS * randn(length(t),1);
            sound(R,samplingrate)
        end

        % Button pushed function: DataanalysisButton
        function DataanalysisButtonPushed(app, event)
            % Get sample size and colormap
            namefile = app.filename;
            app.DialogApp = Loudness_Part_2(app,namefile);
        end

        % Button pushed function: PlayreferencesoundButton
        function PlayreferencesoundButtonPushed(app, event)
            volumen1 = app.Knob.Value+20;
            samplingrate=44100;
            duration=1; %seconds
            t=[0+(1/samplingrate):1/samplingrate:duration]';
            n=length(t);
            freq=(-fix(n/2):fix((n-1)/2))/duration;

            %%%Noise with loudest volume
            L0=volumen1;
            N0=10^-12*(10^(L0/10));%%pp. 48 Hartmann
            CRMS=sqrt(N0*max(freq));%%pp. 48 Hartmann
            R= 0 + CRMS * randn(length(t),1);
            sound(R,samplingrate)
        end
    end

    % Component initialization
    methods (Access = private)

        % Create UIFigure and components
        function createComponents(app)

            % Create UIFigure and hide until all components are created
            app.UIFigure = uifigure('Visible', 'off');
            app.UIFigure.Position = [100 100 693 447];
            app.UIFigure.Name = 'MATLAB App';

            % Create LeftPanel
            app.LeftPanel = uipanel(app.UIFigure);
            app.LeftPanel.Position = [1 12 326 436];

            % Create InitialsEditFieldLabel
            app.InitialsEditFieldLabel = uilabel(app.LeftPanel);
            app.InitialsEditFieldLabel.HorizontalAlignment = 'right';
            app.InitialsEditFieldLabel.Position = [79 130 39 22];
            app.InitialsEditFieldLabel.Text = 'Initials';

            % Create StartExperimentButton
            app.StartExperimentButton = uibutton(app.LeftPanel, 'push');
            app.StartExperimentButton.ButtonPushedFcn = createCallbackFcn(app, @StartExperimentButtonPushed, true);
            app.StartExperimentButton.FontSize = 18;
            app.StartExperimentButton.Position = [89 34 150 31];
            app.StartExperimentButton.Text = 'Start Experiment';

            % Create InitialsEditField
            app.InitialsEditField = uieditfield(app.LeftPanel, 'text');
            app.InitialsEditField.Position = [133 130 75 22];

            % Create AgeEditFieldLabel
            app.AgeEditFieldLabel = uilabel(app.LeftPanel);
            app.AgeEditFieldLabel.HorizontalAlignment = 'right';
            app.AgeEditFieldLabel.Position = [89 87 26 22];
            app.AgeEditFieldLabel.Text = 'Age';

            % Create AgeEditField
            app.AgeEditField = uieditfield(app.LeftPanel, 'text');
            app.AgeEditField.CharacterLimits = [0 150];
            app.AgeEditField.InputType = 'digits';
            app.AgeEditField.Position = [134 87 74 22];

            % Create Knob
            app.Knob = uiknob(app.LeftPanel, 'continuous');
            app.Knob.Limits = [0 50];
            app.Knob.MajorTicks = [0 5 10 15 20 25 30 35 40 45 50];
            app.Knob.MajorTickLabels = {''};
            app.Knob.MinorTicks = [];
            app.Knob.Position = [152 299 81 81];
            app.Knob.Value = 25;

            % Create PlaylowestsoundButton
            app.PlaylowestsoundButton = uibutton(app.LeftPanel, 'push');
            app.PlaylowestsoundButton.ButtonPushedFcn = createCallbackFcn(app, @PlaylowestsoundButtonPushed, true);
            app.PlaylowestsoundButton.Position = [40 232 113 23];
            app.PlaylowestsoundButton.Text = 'Play lowest sound';

            % Create SoundcalibrationLabel
            app.SoundcalibrationLabel = uilabel(app.LeftPanel);
            app.SoundcalibrationLabel.Position = [14 324 116 30];
            app.SoundcalibrationLabel.Text = 'Sound calibration';

            % Create AdjustthevolumesothelowestnoiseisbarelyaudibleLabel
            app.AdjustthevolumesothelowestnoiseisbarelyaudibleLabel = uilabel(app.LeftPanel);
            app.AdjustthevolumesothelowestnoiseisbarelyaudibleLabel.Position = [17 268 302 22];
            app.AdjustthevolumesothelowestnoiseisbarelyaudibleLabel.Text = 'Adjust the volume so the lowest noise is barely audible';

            % Create StevensLoudnessExperimentLabel
            app.StevensLoudnessExperimentLabel = uilabel(app.LeftPanel);
            app.StevensLoudnessExperimentLabel.FontSize = 14;
            app.StevensLoudnessExperimentLabel.FontWeight = 'bold';
            app.StevensLoudnessExperimentLabel.Position = [64 403 212 22];
            app.StevensLoudnessExperimentLabel.Text = 'Stevens'' Loudness Experiment';

            % Create Label
            app.Label = uilabel(app.LeftPanel);
            app.Label.Position = [9 161 309 22];
            app.Label.Text = '__________________________________________________';

            % Create PlayloudestsoundButton
            app.PlayloudestsoundButton = uibutton(app.LeftPanel, 'push');
            app.PlayloudestsoundButton.ButtonPushedFcn = createCallbackFcn(app, @PlayloudestsoundButtonPushed, true);
            app.PlayloudestsoundButton.Position = [176 232 117 23];
            app.PlayloudestsoundButton.Text = 'Play loudest sound';

            % Create SerranoPedrazaI2023Label
            app.SerranoPedrazaI2023Label = uilabel(app.LeftPanel);
            app.SerranoPedrazaI2023Label.FontSize = 9;
            app.SerranoPedrazaI2023Label.Position = [10 0 110 22];
            app.SerranoPedrazaI2023Label.Text = 'Serrano-Pedraza, I. (2023)';

            % Create PlayreferencesoundButton
            app.PlayreferencesoundButton = uibutton(app.LeftPanel, 'push');
            app.PlayreferencesoundButton.ButtonPushedFcn = createCallbackFcn(app, @PlayreferencesoundButtonPushed, true);
            app.PlayreferencesoundButton.Position = [108 187 128 23];
            app.PlayreferencesoundButton.Text = 'Play reference sound';

            % Create RightPanel
            app.RightPanel = uipanel(app.UIFigure);
            app.RightPanel.FontAngle = 'italic';
            app.RightPanel.Position = [326 12 364 436];

            % Create LoudnessestimationEditField
            app.LoudnessestimationEditField = uieditfield(app.RightPanel, 'numeric');
            app.LoudnessestimationEditField.Limits = [0 800];
            app.LoudnessestimationEditField.RoundFractionalValues = 'on';
            app.LoudnessestimationEditField.ValueDisplayFormat = '%.0f';
            app.LoudnessestimationEditField.FontSize = 14;
            app.LoudnessestimationEditField.Enable = 'off';
            app.LoudnessestimationEditField.Position = [216 268 53 22];

            % Create TrialEditFieldLabel
            app.TrialEditFieldLabel = uilabel(app.RightPanel);
            app.TrialEditFieldLabel.BackgroundColor = [0.9412 0.9412 0.9412];
            app.TrialEditFieldLabel.HorizontalAlignment = 'right';
            app.TrialEditFieldLabel.FontSize = 14;
            app.TrialEditFieldLabel.Position = [75 340 30 22];
            app.TrialEditFieldLabel.Text = 'Trial';

            % Create TrialEditField
            app.TrialEditField = uieditfield(app.RightPanel, 'numeric');
            app.TrialEditField.FontSize = 14;
            app.TrialEditField.BackgroundColor = [0.9412 0.9412 0.9412];
            app.TrialEditField.Position = [125 340 31 22];
            app.TrialEditField.Value = 1;

            % Create RunningLampLabel
            app.RunningLampLabel = uilabel(app.RightPanel);
            app.RunningLampLabel.HorizontalAlignment = 'right';
            app.RunningLampLabel.FontSize = 14;
            app.RunningLampLabel.Position = [207 340 57 22];
            app.RunningLampLabel.Text = 'Running';

            % Create RunningLamp
            app.RunningLamp = uilamp(app.RightPanel);
            app.RunningLamp.Position = [279 340 23 23];
            app.RunningLamp.Color = [1 0 0];

            % Create NexttrialButton
            app.NexttrialButton = uibutton(app.RightPanel, 'push');
            app.NexttrialButton.ButtonPushedFcn = createCallbackFcn(app, @NexttrialButtonPushed, true);
            app.NexttrialButton.FontSize = 14;
            app.NexttrialButton.Enable = 'off';
            app.NexttrialButton.Position = [153 203 100 25];
            app.NexttrialButton.Text = 'Next trial';

            % Create RestartButton
            app.RestartButton = uibutton(app.RightPanel, 'push');
            app.RestartButton.ButtonPushedFcn = createCallbackFcn(app, @RestartButtonPushed, true);
            app.RestartButton.FontSize = 14;
            app.RestartButton.Position = [171 40 64 25];
            app.RestartButton.Text = 'Restart';

            % Create of27Label
            app.of27Label = uilabel(app.RightPanel);
            app.of27Label.Position = [164 340 32 22];
            app.of27Label.Text = 'of 27';

            % Create Label_2
            app.Label_2 = uilabel(app.RightPanel);
            app.Label_2.FontSize = 14;
            app.Label_2.Enable = 'off';
            app.Label_2.Position = [279 267 49 22];
            app.Label_2.Text = '(1-800)';

            % Create DataanalysisButton
            app.DataanalysisButton = uibutton(app.RightPanel, 'push');
            app.DataanalysisButton.ButtonPushedFcn = createCallbackFcn(app, @DataanalysisButtonPushed, true);
            app.DataanalysisButton.FontSize = 18;
            app.DataanalysisButton.FontWeight = 'bold';
            app.DataanalysisButton.FontColor = [0 0.4471 0.7412];
            app.DataanalysisButton.Enable = 'off';
            app.DataanalysisButton.Visible = 'off';
            app.DataanalysisButton.Position = [137 113 130 31];
            app.DataanalysisButton.Text = 'Data analysis';

            % Create LoudnessestimationEditFieldLabel_2
            app.LoudnessestimationEditFieldLabel_2 = uilabel(app.RightPanel);
            app.LoudnessestimationEditFieldLabel_2.HorizontalAlignment = 'right';
            app.LoudnessestimationEditFieldLabel_2.FontSize = 14;
            app.LoudnessestimationEditFieldLabel_2.Enable = 'off';
            app.LoudnessestimationEditFieldLabel_2.Position = [75 267 135 22];
            app.LoudnessestimationEditFieldLabel_2.Text = 'Loudness estimation';

            % Create Label_3
            app.Label_3 = uilabel(app.RightPanel);
            app.Label_3.WordWrap = 'on';
            app.Label_3.Enable = 'off';
            app.Label_3.Position = [10 126 116 129];
            app.Label_3.Text = 'Note: The reference stimulus is always presented first. The Loudness estimation for the reference stimulus is 100.';

            % Show the figure after all components are created
            app.UIFigure.Visible = 'on';
        end
    end

    % App creation and deletion
    methods (Access = public)

        % Construct app
        function app = Experiment_Stevens_Loudness

            % Create UIFigure and components
            createComponents(app)

            % Register the app with App Designer
            registerApp(app, app.UIFigure)

            if nargout == 0
                clear app
            end
        end

        % Code that executes before app deletion
        function delete(app)

            % Delete UIFigure when app is deleted
            delete(app.UIFigure)
        end
    end
end