classdef Main_panel_2023 < matlab.apps.AppBase

    % Properties that correspond to app components
    properties (Access = public)
        UIFigure                       matlab.ui.Figure
        GridLayout                     matlab.ui.container.GridLayout
        LeftPanel                      matlab.ui.container.Panel
        ExperimentDisparitySensitivityButton  matlab.ui.control.Button
        ExperimentContrastSensitivityButton  matlab.ui.control.Button
        ExperimentCriticalBandsButton  matlab.ui.control.Button
        ExperimentAudibilityCurveButton  matlab.ui.control.Button
        ExperimentBrightnessButton     matlab.ui.control.Button
        ExperimentLoudnessButton       matlab.ui.control.Button
        EXPERIMENTSLabel               matlab.ui.control.Label
        SerranoPedrazaI2023Label       matlab.ui.control.Label
        RightPanel                     matlab.ui.container.Panel
        AnalysisDisparitySensitivityFunctionButton  matlab.ui.control.Button
        AnalysisColorimetryButton      matlab.ui.control.Button
        AnalysisContrastSensitivityFunctionButton  matlab.ui.control.Button
        AnalysisCriticalBandsButton    matlab.ui.control.Button
        AnalysisLoudnessButton         matlab.ui.control.Button
        AnalysisBrightnessButton       matlab.ui.control.Button
        ANALYSISLabel                  matlab.ui.control.Label
    end

    % Properties that correspond to apps with auto-reflow
    properties (Access = private)
        onePanelWidth = 576;
    end

    
    properties (Access = private)
        Experiment;
    end
    
    methods (Access = private)
        
        function updateplot(app)

            %%%Dr. Ignacio Serrano-Pedraza 2023
            %%%With this program you can execute all programs (experiments and the
            %%%analysis of the experiments).
            %%%%A detailed explanation of the experiments can be seen here:
            % % Serrano-Pedraza, I., Sierra-Vázquez, V., & López-Báscuas, L.E. (2014). 
            % % Psicología de la Percepción. Prácticas. Ed. Síntesis. ISBN: 978-84-907704-6-7


            if app.Experiment==1
                Experiment_Stevens_Loudness;
            end
            if app.Experiment==2
                Experiment_Stevens_Brightness;
            end
            if app.Experiment==3
                Experiment_Critical_Bands;
             end
             if app.Experiment==4
                Experiment_Audibility_curve;
             end
             if app.Experiment==5
                AnalysisLoudness;
             end
             if app.Experiment==6
                AnalysisBrightness;
             end
             if app.Experiment==7
                Analysis_Critical_Bands;
             end
             if app.Experiment==8
                Analysis_Contrast_Sensitivity_Function;
             end
             if app.Experiment==9
                Colorimetry_light;
             end
             if app.Experiment==10
                Experiment_CSF;
             end
             if app.Experiment==11
                Experiment_DSF_3D;
             end
             if app.Experiment==12
                Analysis_Disparity_Sensitivity_Function;
             end
        end
    end
    

    % Callbacks that handle component events
    methods (Access = private)

        % Button pushed function: ExperimentBrightnessButton
        function ExperimentBrightnessButtonPushed(app, event)
            app.Experiment=2;
            app.updateplot;
        end

        % Button pushed function: ExperimentLoudnessButton
        function ExperimentLoudnessButtonPushed(app, event)
            app.Experiment=1;
            app.updateplot;
        end

        % Button pushed function: ExperimentCriticalBandsButton
        function ExperimentCriticalBandsButtonPushed(app, event)
            app.Experiment=3;
            app.updateplot;
        end

        % Button pushed function: ExperimentAudibilityCurveButton
        function ExperimentAudibilityCurveButtonPushed(app, event)
            app.Experiment=4;
            app.updateplot;
        end

        % Button pushed function: AnalysisLoudnessButton
        function AnalysisLoudnessButtonPushed(app, event)
            app.Experiment=5;
            app.updateplot; 
        end

        % Button pushed function: AnalysisBrightnessButton
        function AnalysisBrightnessButtonPushed(app, event)
            app.Experiment=6;
            app.updateplot;
        end

        % Button pushed function: AnalysisCriticalBandsButton
        function AnalysisCriticalBandsButtonPushed(app, event)
            app.Experiment=7;
            app.updateplot;
        end

        % Button pushed function: AnalysisContrastSensitivityFunctionButton
        function AnalysisContrastSensitivityFunctionButtonPushed(app, event)
            app.Experiment=8;
            app.updateplot;
        end

        % Button pushed function: AnalysisColorimetryButton
        function AnalysisColorimetryButtonPushed(app, event)
            app.Experiment=9;
            app.updateplot;
        end

        % Button pushed function: ExperimentContrastSensitivityButton
        function ExperimentContrastSensitivityButtonPushed(app, event)
            app.Experiment=10;
            app.updateplot;
        end

        % Button pushed function: ExperimentDisparitySensitivityButton
        function ExperimentDisparitySensitivityButtonPushed(app, event)
            app.Experiment=11;
            app.updateplot;
        end

        % Button pushed function: 
        % AnalysisDisparitySensitivityFunctionButton
        function AnalysisDisparitySensitivityFunctionButtonPushed(app, event)
            app.Experiment=12;
            app.updateplot;
        end

        % Changes arrangement of the app based on UIFigure width
        function updateAppLayout(app, event)
            currentFigureWidth = app.UIFigure.Position(3);
            if(currentFigureWidth <= app.onePanelWidth)
                % Change to a 2x1 grid
                app.GridLayout.RowHeight = {394, 394};
                app.GridLayout.ColumnWidth = {'1x'};
                app.RightPanel.Layout.Row = 2;
                app.RightPanel.Layout.Column = 1;
            else
                % Change to a 1x2 grid
                app.GridLayout.RowHeight = {'1x'};
                app.GridLayout.ColumnWidth = {284, '1x'};
                app.RightPanel.Layout.Row = 1;
                app.RightPanel.Layout.Column = 2;
            end
        end
    end

    % Component initialization
    methods (Access = private)

        % Create UIFigure and components
        function createComponents(app)

            % Create UIFigure and hide until all components are created
            app.UIFigure = uifigure('Visible', 'off');
            app.UIFigure.AutoResizeChildren = 'off';
            app.UIFigure.Position = [100 100 594 394];
            app.UIFigure.Name = 'MATLAB App';
            app.UIFigure.SizeChangedFcn = createCallbackFcn(app, @updateAppLayout, true);

            % Create GridLayout
            app.GridLayout = uigridlayout(app.UIFigure);
            app.GridLayout.ColumnWidth = {284, '1x'};
            app.GridLayout.RowHeight = {'1x'};
            app.GridLayout.ColumnSpacing = 0;
            app.GridLayout.RowSpacing = 0;
            app.GridLayout.Padding = [0 0 0 0];
            app.GridLayout.Scrollable = 'on';

            % Create LeftPanel
            app.LeftPanel = uipanel(app.GridLayout);
            app.LeftPanel.Layout.Row = 1;
            app.LeftPanel.Layout.Column = 1;

            % Create SerranoPedrazaI2023Label
            app.SerranoPedrazaI2023Label = uilabel(app.LeftPanel);
            app.SerranoPedrazaI2023Label.FontSize = 9;
            app.SerranoPedrazaI2023Label.Position = [21 11 110 22];
            app.SerranoPedrazaI2023Label.Text = 'Serrano-Pedraza, I. (2023)';

            % Create EXPERIMENTSLabel
            app.EXPERIMENTSLabel = uilabel(app.LeftPanel);
            app.EXPERIMENTSLabel.FontSize = 14;
            app.EXPERIMENTSLabel.FontWeight = 'bold';
            app.EXPERIMENTSLabel.Position = [89 343 106 22];
            app.EXPERIMENTSLabel.Text = 'EXPERIMENTS';

            % Create ExperimentLoudnessButton
            app.ExperimentLoudnessButton = uibutton(app.LeftPanel, 'push');
            app.ExperimentLoudnessButton.ButtonPushedFcn = createCallbackFcn(app, @ExperimentLoudnessButtonPushed, true);
            app.ExperimentLoudnessButton.Position = [61 308 161 23];
            app.ExperimentLoudnessButton.Text = 'Experiment Loudness';

            % Create ExperimentBrightnessButton
            app.ExperimentBrightnessButton = uibutton(app.LeftPanel, 'push');
            app.ExperimentBrightnessButton.ButtonPushedFcn = createCallbackFcn(app, @ExperimentBrightnessButtonPushed, true);
            app.ExperimentBrightnessButton.Position = [61 261 161 23];
            app.ExperimentBrightnessButton.Text = 'Experiment Brightness';

            % Create ExperimentAudibilityCurveButton
            app.ExperimentAudibilityCurveButton = uibutton(app.LeftPanel, 'push');
            app.ExperimentAudibilityCurveButton.ButtonPushedFcn = createCallbackFcn(app, @ExperimentAudibilityCurveButtonPushed, true);
            app.ExperimentAudibilityCurveButton.Position = [59 162 163 23];
            app.ExperimentAudibilityCurveButton.Text = 'Experiment Audibility Curve';

            % Create ExperimentCriticalBandsButton
            app.ExperimentCriticalBandsButton = uibutton(app.LeftPanel, 'push');
            app.ExperimentCriticalBandsButton.ButtonPushedFcn = createCallbackFcn(app, @ExperimentCriticalBandsButtonPushed, true);
            app.ExperimentCriticalBandsButton.Position = [59 211 163 23];
            app.ExperimentCriticalBandsButton.Text = 'Experiment Critical Bands';

            % Create ExperimentContrastSensitivityButton
            app.ExperimentContrastSensitivityButton = uibutton(app.LeftPanel, 'push');
            app.ExperimentContrastSensitivityButton.ButtonPushedFcn = createCallbackFcn(app, @ExperimentContrastSensitivityButtonPushed, true);
            app.ExperimentContrastSensitivityButton.Position = [54 113 183 23];
            app.ExperimentContrastSensitivityButton.Text = 'Experiment Contrast Sensitivity';

            % Create ExperimentDisparitySensitivityButton
            app.ExperimentDisparitySensitivityButton = uibutton(app.LeftPanel, 'push');
            app.ExperimentDisparitySensitivityButton.ButtonPushedFcn = createCallbackFcn(app, @ExperimentDisparitySensitivityButtonPushed, true);
            app.ExperimentDisparitySensitivityButton.Position = [54 65 184 23];
            app.ExperimentDisparitySensitivityButton.Text = 'Experiment Disparity Sensitivity';

            % Create RightPanel
            app.RightPanel = uipanel(app.GridLayout);
            app.RightPanel.Layout.Row = 1;
            app.RightPanel.Layout.Column = 2;

            % Create ANALYSISLabel
            app.ANALYSISLabel = uilabel(app.RightPanel);
            app.ANALYSISLabel.FontSize = 14;
            app.ANALYSISLabel.FontWeight = 'bold';
            app.ANALYSISLabel.Position = [122 343 73 22];
            app.ANALYSISLabel.Text = 'ANALYSIS';

            % Create AnalysisBrightnessButton
            app.AnalysisBrightnessButton = uibutton(app.RightPanel, 'push');
            app.AnalysisBrightnessButton.ButtonPushedFcn = createCallbackFcn(app, @AnalysisBrightnessButtonPushed, true);
            app.AnalysisBrightnessButton.Position = [78 261 161 23];
            app.AnalysisBrightnessButton.Text = 'Analysis Brightness';

            % Create AnalysisLoudnessButton
            app.AnalysisLoudnessButton = uibutton(app.RightPanel, 'push');
            app.AnalysisLoudnessButton.ButtonPushedFcn = createCallbackFcn(app, @AnalysisLoudnessButtonPushed, true);
            app.AnalysisLoudnessButton.Position = [78 308 161 23];
            app.AnalysisLoudnessButton.Text = 'Analysis Loudness';

            % Create AnalysisCriticalBandsButton
            app.AnalysisCriticalBandsButton = uibutton(app.RightPanel, 'push');
            app.AnalysisCriticalBandsButton.ButtonPushedFcn = createCallbackFcn(app, @AnalysisCriticalBandsButtonPushed, true);
            app.AnalysisCriticalBandsButton.Position = [78 211 161 23];
            app.AnalysisCriticalBandsButton.Text = 'Analysis Critical Bands';

            % Create AnalysisContrastSensitivityFunctionButton
            app.AnalysisContrastSensitivityFunctionButton = uibutton(app.RightPanel, 'push');
            app.AnalysisContrastSensitivityFunctionButton.ButtonPushedFcn = createCallbackFcn(app, @AnalysisContrastSensitivityFunctionButtonPushed, true);
            app.AnalysisContrastSensitivityFunctionButton.Position = [50 162 218 23];
            app.AnalysisContrastSensitivityFunctionButton.Text = 'Analysis Contrast Sensitivity Function';

            % Create AnalysisColorimetryButton
            app.AnalysisColorimetryButton = uibutton(app.RightPanel, 'push');
            app.AnalysisColorimetryButton.ButtonPushedFcn = createCallbackFcn(app, @AnalysisColorimetryButtonPushed, true);
            app.AnalysisColorimetryButton.Position = [78 65 161 23];
            app.AnalysisColorimetryButton.Text = 'Analysis Colorimetry';

            % Create AnalysisDisparitySensitivityFunctionButton
            app.AnalysisDisparitySensitivityFunctionButton = uibutton(app.RightPanel, 'push');
            app.AnalysisDisparitySensitivityFunctionButton.ButtonPushedFcn = createCallbackFcn(app, @AnalysisDisparitySensitivityFunctionButtonPushed, true);
            app.AnalysisDisparitySensitivityFunctionButton.Position = [50 113 218 23];
            app.AnalysisDisparitySensitivityFunctionButton.Text = 'Analysis Disparity Sensitivity Function';

            % Show the figure after all components are created
            app.UIFigure.Visible = 'on';
        end
    end

    % App creation and deletion
    methods (Access = public)

        % Construct app
        function app = Main_panel_2023

            % Create UIFigure and components
            createComponents(app)

            % Register the app with App Designer
            registerApp(app, app.UIFigure)

            if nargout == 0
                clear app
            end
        end

        % Code that executes before app deletion
        function delete(app)

            % Delete UIFigure when app is deleted
            delete(app.UIFigure)
        end
    end
end